<?php

namespace Drupal\list_packages\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\testsuite\BaseTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for dblog routes.
 */
class ListDashboardController extends ControllerBase {
  use BaseTrait;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('database'),
      $container->get('date.formatter'),
      $container->get('form_builder'),
    );
  }

  /**
   * Constructs a DbLogController object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    Connection $database,
    DateFormatterInterface $date_formatter,
    FormBuilderInterface $form_builder,
  ) {
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->dateFormatter = $date_formatter;
    $this->formBuilder = $form_builder;
  }

  /**
   * Displays a listing of database log messages.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   */
  public function overview() {

    $this->moduleHandler()->loadInclude('list_packages', 'admin.inc');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_dashboard', 'lpd');
    $query->fields(
      'lpd',
      [
        'id',
        'core_library_report',
        'packages_report',
        'libraries_report',
        'modules_report',
        'themes_report',
        'modsthems_ext_css_libraries_report',
        'modsthems_ext_js_libraries_report',
        'created',
      ]
    );
    $dashboard = $query->execute()->fetchAssoc();
    $rows = [];

    $area = [
      'core libraries',
      'packages',
      'libraries',
      'modules & themes',
      'modules & themes Css Libraries',
      'modules & themes Js Libraries',
    ];
    $cards = [];
    foreach ($area as $index => $card) {
      $report = "";
      $formElement = [];
      switch ($index) {
        case 0:
          $formElement['list_core_libraries_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\LoadCoreLibrariesForm');
          if ($dashboard) {
            if ($dashboard['core_library_report']) {
              $report = '<p>' . $dashboard['core_library_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Core Libraries button to load libraires.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Core Libraries button to load libraires.<p>';
          }
          break;

        case 1:
          $formElement['list_packages_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\LoadPackagesForm');
          if ($dashboard) {
            if ($dashboard['packages_report']) {
              $report = '<p>' . $dashboard['packages_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Packages button to load packages.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Packages button to load packages.<p>';
          }
          break;

        case 2:
          $formElement['list_libraries_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\LoadLibrariesForm');
          if ($dashboard) {
            if ($dashboard['libraries_report']) {
              $report = '<p>' . $dashboard['libraries_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Libraries button to load packages.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Libraries button to load packages.<p>';
          }
          break;

        case 3:
          $formElement['list_mod_thems_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\LoadModsThemsForm');
          if ($dashboard) {
            if ($dashboard['modules_report'] || $dashboard['themes_report']) {
              $report = '<p>' . $dashboard['modules_report'] . $dashboard['themes_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
          }
          break;

        case 4:
          if ($dashboard) {
            if ($dashboard['modsthems_ext_css_libraries_report']) {
              $report = '<p>' . $dashboard['modsthems_ext_css_libraries_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
          }
          break;

        case 5:
          if ($dashboard) {
            if ($dashboard['modsthems_ext_js_libraries_report']) {
              $report = '<p>' . $dashboard['modsthems_ext_js_libraries_report'] . '</p>';
            }
            else {
              $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
            }
          }
          else {
            $report = '<p>Hit the Load Modules & Themes button to load modules themes and their libraries.<p>';
          }
          break;
      }
      $cards[] = [
        '#type' => 'container',
        '#attributes' => [
          'class' => [
            'system-status-general-info__item',
            'card',
          ],
        ],
        '#children' => [
          [
            '#type' => 'markup',
            '#markup' => '<span class="system-status-general-info__item-icon system-status-general-info__item-icon--drupal"></span>',
          ],
          [
            '#type' => 'container',
            '#attributes' => [
              'class' => [
                'system-status-general-info__item-details',
              ],
            ],
            '#children' => [
              [
                '#type' => 'markup',
                '#markup' => '<h3 class="system-status-general-info__item-title">' . ucwords($card) . '</h3>' . $report,
              ],
              $formElement,
            ],
          ],
        ],
      ];
    }

    $packagistLink = Link::fromTextAndUrl(
      $this->t('packagist.org'),
      Url::fromUri('https://packagist.org/')
    )->toString();

    $npmLink = Link::fromTextAndUrl(
      $this->t('registry.npmjs.org'),
      Url::fromUri('https://registry.npmjs.org/')
    )->toString();

    $drupalLink = Link::fromTextAndUrl(
      $this->t('packages.drupal.org'),
      Url::fromUri('https://packages.drupal.org')
    )->toString();

    $githubLink = Link::fromTextAndUrl(
      $this->t('api.github.com'),
      Url::fromUri('https://api.github.com')
    )->toString();

    $polyfillLink = Link::fromTextAndUrl(
      $this->t('polyfill.io'),
      Url::fromUri('https://polyfill.io')
    )->toString();

    $cdns = Link::fromTextAndUrl(
      $this->t('Content Delivery Network (CDN)'),
      Url::fromUri('https://en.wikipedia.org/wiki/Content_delivery_network')
    )->toString();

    $build = $this->createLegend();

    $build['inner_dashboard']['upper_dashboard'] = [
      '#type' => 'container',
      '#children' => [
        [
          '#type' => 'markup',
          '#markup' => '<p><span class="testsuite-red">This package checker is not 100% reliable to catch every single vulnerability. I take no responsibility for wrong data. It could return false positives or not find vulnerabilities that are reported on the software. Double check all results.</span><br />This package checker calls these API endpoints ' . $packagistLink . ', ' . $npmLink . ', ' . $githubLink . ' and ' . $drupalLink . ' to check packages, libraries, modules and themes for vulnerabilities. <span class="testsuite-red">The public GitHub API requests are limited to 60 per hour per ip.</span><br />There are require and require-dev packages in composer.json and dependencies, peerDependencies and devDependencies in Npm packages that are not checked but seem to be covered by scanning the parent packages. For deeper security you could also want to scan all of these.<br />There are also external scripts that are comming in from ' . $polyfillLink . ' and ' . $cdns . ' that are not checked yet.<br />Any none php or js packages that are not registered with the first three links are not checked. To get the most out of this tool if the package does not use these, see if your developer can switch the packages out for packages that use them.<br />If the version in the table says ANY then the version is missing in the package.json or the composer.json file. When I check and get a result and the version is ANY it will return matches to all versions. You will need to search through the installed package to find out what version you have installed. Sometimes you can find it at the top of the css or js file. Sometimes the software will include a VERSION.txt file.</p>',
        ],
      ],
    ];

    $libraries = [
      'testsuite/testsuite.filters',
      'testsuite/testsuite.claro',
    ];
    $adminTheme = $this->configFactory->get('system.theme')->get('admin');
    if ($adminTheme == "seven") {
      $libraries = [
        'testsuite/testsuite.filters',
        'testsuite/testsuite.seven',
      ];
    }

    $build['dashboard'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => [
          'system-status-general-info',
          'testsuite-margin-bottom-10',
        ],
      ],
      '#attached' => [
        'library' => $libraries,
      ],
      '#children' => [
        [
          '#type' => 'markup',
          '#markup' => '<h2 class="system-status-general-info__header">List Packages & Vulnerability Checker</h2>',
        ],
        [
          '#type' => 'container',
          '#attributes' => [
            'class' => [
              'system-status-general-info__items',
            ],
          ],
          '#children' => $cards,
        ],
      ],
    ];

    $header = [
      [
        'data' => $this->t('Date Created'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Name'),
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Type'),
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Version'),
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Latest'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Description'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerabilities'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerability Count'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
    ];

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_vulnerabilities', 'lpv');
    $query->fields(
      'lpv', [
        'id',
        'table',
        'foreign_key_id',
      ]
    );
    $query->fields(
      'lpcl', [
        'name',
        'type',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );
    $query->leftJoin('lp_core_library', 'lpcl', '[lpcl].[id] = [lpv].[foreign_key_id]');
    $query->condition('lpv.table', $this->databases['lp_core_library']);
    $query->condition('lpcl.if_vulnerability', [0, 1, 2], 'IN');
    $query->orderBy('lpcl.if_vulnerability');

    $coreLibrariesResults = $query->execute()->fetchAllAssoc('id');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_vulnerabilities', 'lpv');
    $query->fields(
      'lpv', [
        'id',
        'table',
        'foreign_key_id',
      ]
    );
    $query->fields(
      'lpip', [
        'name',
        'type',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );
    $query->leftJoin('lp_installed_package', 'lpip', '[lpip].[id] = [lpv].[foreign_key_id]');
    $query->condition('lpip.if_vulnerability', [0, 1, 2], 'IN');
    $query->orderBy('lpip.if_vulnerability');

    $packageResults = $query->execute()->fetchAllAssoc('id');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_vulnerabilities', 'lpv');
    $query->fields(
      'lpv', [
        'id',
        'table',
        'foreign_key_id',
      ]
    );
    $query->fields(
      'lpil', [
        'name',
        'type',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );
    $query->leftJoin('lp_installed_library', 'lpil', '[lpil].[id] = [lpv].[foreign_key_id]');
    $query->condition('lpv.table', $this->databases['lp_installed_library']);
    $query->condition('lpil.if_vulnerability', [0, 1, 2], 'IN');
    $query->orderBy('lpil.if_vulnerability');

    $librariesResults = $query->execute()->fetchAllAssoc('id');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_vulnerabilities', 'lpv');
    $query->fields(
      'lpv', [
        'id',
        'table',
        'foreign_key_id',
      ]
    );
    $query->fields(
      'lpim', [
        'name',
        'type',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );
    $query->leftJoin('lp_installed_module', 'lpim', '[lpim].[id] = [lpv].[foreign_key_id]');
    $query->condition('lpv.table', $this->databases['lp_installed_module']);
    $query->condition('lpim.if_vulnerability', [0, 1, 2], 'IN');
    $query->orderBy('lpim.if_vulnerability');

    $moduleResults = $query->execute()->fetchAllAssoc('id');

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_vulnerabilities', 'lpv');
    $query->fields(
      'lpv', [
        'id',
        'table',
        'foreign_key_id',
      ]
    );
    $query->fields(
      'lpml', [
        'name',
        'type',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );
    $query->leftJoin('lp_module_library', 'lpml', '[lpml].[id] = [lpv].[foreign_key_id]');
    $query->condition('lpv.table', $this->databases['lp_module_library']);
    $query->condition('lpml.if_vulnerability', [0, 1, 2], 'IN');
    $query->orderBy('lpml.if_vulnerability');

    $moduleLibraryResults = $query->execute()->fetchAllAssoc('id');

    $results = array_merge(
      $coreLibrariesResults,
      $packageResults,
      $librariesResults,
      $moduleResults,
      $moduleLibraryResults
    );

    usort($results, 'comparator');

    foreach ($results as $testItem) {
      switch ($testItem->table) {
        case 0:
          $route = 'list_packages.installed_core_libraries_event';
          break;

        case 1:
          $route = 'list_packages.installed_packages_event';
          break;

        case 2:
          $route = 'list_packages.installed_libraries_event';
          break;

        case 3:
          $route = 'list_packages.installed_modules_event';
          break;

        case 4:
          $route = 'list_packages.installed_modules_libraries_event';
          break;
      }

      $name = Link::fromTextAndUrl(
        Markup::create(Xss::filterAdmin($testItem->name)), new Url(
          $route, ['event_id' => $testItem->foreign_key_id], [
            'attributes' => [
              'title' => Xss::filter($testItem->name),
            ],
          ]
        )
      )->toString();

      $rows[] = [
        'data' => [
          $this->dateFormatter->format($testItem->created, 'short'),
          $name,
          Xss::filter($testItem->type),
          Xss::filter($testItem->version),
          Xss::filter($testItem->latest),
          Markup::create(Xss::filterAdmin($testItem->description)),
          Markup::create($this->getEmoji($testItem->if_vulnerability)),
          Xss::filter($testItem->vulnerability_count),
        ],
      ];
    }

    $build['list_vulnerabilitites_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
    ];

    return $build;

  }

}
