<?php

namespace Drupal\list_packages\Controller;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Database\Query\TableSortExtender;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\list_packages\GetDrupalClient;
use Drupal\testsuite\BaseTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Returns responses for dblog routes.
 */
class ListModulesController extends ControllerBase {
  use BaseTrait;

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The user storage.
   *
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * The NPM Registery client.
   *
   * @var \Drupal\list_packages\GetDrupalClient
   */
  protected $getDrupalClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('file_system'),
      $container->get('module_handler'),
      $container->get('date.formatter'),
      $container->get('form_builder'),
      $container->get('list_packages.drupal_client.service')
    );
  }

  /**
   * Constructs a DbLogController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   A database connection.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   A module handler.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\list_packages\GetDrupalClient $getDrupalClient
   *   Get package.drupal info on a package.
   */
  public function __construct(
    Connection $database,
    FileSystemInterface $file_system,
    ModuleHandlerInterface $module_handler,
    DateFormatterInterface $date_formatter,
    FormBuilderInterface $form_builder,
    GetDrupalClient $getDrupalClient,
  ) {
    $this->database = $database;
    $this->fileSystem = $file_system;
    $this->moduleHandler = $module_handler;
    $this->dateFormatter = $date_formatter;
    $this->formBuilder = $form_builder;
    $this->getDrupalClient = $getDrupalClient;
    $this->userStorage = $this->entityTypeManager()->getStorage('user');
  }

  /**
   * Displays a listing of database log messages.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   *
   * @see Drupal\dblog\Form\DblogClearLogConfirmForm
   * @see Drupal\dblog\Controller\DbLogController::eventDetails()
   */
  public function overview(Request $request) {
    $filter = $this->buildModulesFilterQuery($request);
    $rows = [];

    $description = 'This is the report of all the Modules and Themes in the Core, Contrib and Custom folders of your Drupal installation.';
    $build = $this->createLegend($description);

    $build['list_modules_filter_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\ListModulesFilterForm');
    $header = [
      [
        'data' => $this->t('Date Created'),
        'field' => 'lpip.created',
        'sort' => 'desc',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Name'),
        'field' => 'lpip.name',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Type'),
        'field' => 'lpip.type',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Version'),
        'field' => 'lpip.version',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Latest'),
        'field' => 'lpip.latest',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Description'),
        'field' => 'lpip.description',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerabilities'),
        'field' => 'lpip.if_vulnerability',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerability Count'),
        'field' => 'lpip.vulnerability_count',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
    ];

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_installed_module', 'lpip')
      ->extend(PagerSelectExtender::class)
      ->extend(TableSortExtender::class);
    $query->fields(
      'lpip', [
        'id',
        'type',
        'name',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );

    if (!empty($filter['where'])) {
      $query->where($filter['where'], $filter['args']);
    }

    $result = $query
      ->limit(50)
      ->orderByHeader($header)
      ->execute();

    foreach ($result as $testItem) {
      $name = Link::fromTextAndUrl(
        Xss::filter($testItem->name), new Url(
          'list_packages.installed_modules_event', ['event_id' => $testItem->id], [
            'attributes' => [
              'title' => Xss::filter($testItem->name),
            ],
          ]
        )
      )->toString();
      $rows[] = [
        'data' => [
          $this->dateFormatter->format($testItem->created, 'short'),
          $name,
          $this->createName($testItem->type),
          Xss::filter($testItem->version),
          Xss::filter($testItem->latest),
          Xss::filter($testItem->description),
          Markup::create($this->getEmoji($testItem->if_vulnerability)),
          Xss::filter($testItem->vulnerability_count),
        ],
      ];
    }

    $build['list_packages_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
    ];

    $build['list_packages_pager'] = ['#type' => 'pager'];

    return $build;

  }

  /**
   * Turns id into a name.
   *
   * @param string $type
   *   The id styled text.
   *
   * @return string
   *   A name delimeted with spaces.
   */
  private function createName($type) {
    if ($type != "") {
      return ucwords(implode(" ", explode("-", $type)));
    }
  }

  /**
   * Displays a listing of database log messages.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   *
   * @see Drupal\dblog\Form\DblogClearLogConfirmForm
   * @see Drupal\dblog\Controller\DbLogController::eventDetails()
   */
  public function moduleLibraries(Request $request) {
    $filter = $this->buildModuleLibrariesFilterQuery($request);
    $rows = [];

    $this->moduleHandler()->loadInclude('list_packages', 'admin.inc');

    $description = 'This is the report of all the css and js files used by Modules and Themes in the Core, Contrib and Custom folders of your Drupal installation.';
    $build = $this->createLegend($description);

    $build['list_modules_libraries_filter_form'] = $this->formBuilder()->getForm('Drupal\list_packages\Form\ListModuleLibrariesFilterForm');
    $header = [
      [
        'data' => $this->t('Date Created'),
        'field' => 'lml.created',
        'sort' => 'desc',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Module Name'),
        'field' => 'lpim.name',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Libraries Name'),
        'field' => 'lml.name',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Type'),
        'field' => 'lml.type',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('If Internal'),
        'field' => 'lml.if_internal',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Version'),
        'field' => 'lml.version',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Latest'),
        'field' => 'lml.latest',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Description'),
        'field' => 'lml.description',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerabilities'),
        'field' => 'lml.if_vulnerability',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
      [
        'data' => $this->t('Vulnerability Count'),
        'field' => 'lml.vulnerability_count',
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
    ];

    /** @var \Drupal\Core\Database\Query\SelectInterface $query */
    $query = $this->database->select('lp_module_library', 'lml')
      ->extend(PagerSelectExtender::class)
      ->extend(TableSortExtender::class);

    $query->fields(
        'lpim', [
          'name',
        ]
    );
    $query->fields(
      'lml', [
        'id',
        'lp_installed_module_id',
        'type',
        'if_internal',
        'name',
        'version',
        'latest',
        'description',
        'if_vulnerability',
        'vulnerability_count',
        'created',
      ]
    );

    $query->leftJoin('lp_installed_module', 'lpim', '[lpim].[id] = [lml].[lp_installed_module_id]');

    // dd($filter);
    if (!empty($filter['where'])) {
      $query->where($filter['where'], $filter['args']);
    }

    $results = $query
      ->limit(50)
      ->orderByHeader($header)
      ->execute()->fetchAllAssoc('id');

    usort($results, 'comparator');

    foreach ($results as $testItem) {
      // dd($testItem);
      $name = Link::fromTextAndUrl(
        Xss::filter($testItem->name), new Url(
          'list_packages.installed_modules_event', ['event_id' => $testItem->lp_installed_module_id], [
            'attributes' => [
              'title' => Xss::filter($testItem->name),
            ],
          ]
        )
      )->toString();
      $lml_name = Link::fromTextAndUrl(
        Xss::filter($testItem->lml_name), new Url(
          'list_packages.installed_modules_libraries_event', ['event_id' => $testItem->id], [
            'attributes' => [
              'title' => Xss::filter($testItem->lml_name),
            ],
          ]
        )
      )->toString();
      $rows[] = [
        'data' => [
          $this->dateFormatter->format($testItem->created, 'short'),
          $name,
          $lml_name,
          $testItem->type,
          $testItem->if_internal ? "Internal" : "External",
          Xss::filter($testItem->version),
          Xss::filter($testItem->latest),
          Xss::filter($testItem->description),
          Markup::create($this->getEmoji($testItem->if_vulnerability)),
          Xss::filter($testItem->vulnerability_count),
        ],
      ];
    }

    $build['list_packages_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No log messages available.'),
    ];

    $build['list_packages_pager'] = ['#type' => 'pager'];

    return $build;
  }

  /**
   * Displays details about a specific database log message.
   *
   * @param int $event_id
   *   Unique ID of the database log message.
   *
   * @return array
   *   If the ID is located in the Database Logging table, a build array in the
   *   format expected by \Drupal\Core\Render\RendererInterface::render().
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If no event found for the given ID.
   */
  public function eventDetails($event_id) {
    $testItem = $this->database->query('SELECT [lpip].* FROM {lp_installed_module} [lpip] WHERE [lpip].[id] = :id', [':id' => $event_id])->fetchObject();

    if (empty($testItem)) {
      throw new NotFoundHttpException();
    }

    $build = [];
    $rows = [
      [
      ['data' => $this->t('Date Created'), 'header' => TRUE],
        $this->dateFormatter->format($testItem->created, 'long'),
      ],
      [
        ['data' => $this->t('Name'), 'header' => TRUE],
        Xss::filter($testItem->name),
      ],
      [
      ['data' => $this->t('Type'), 'header' => TRUE],
        $testItem->type,
      ],
      [
      ['data' => $this->t('Vendor'), 'header' => TRUE],
        Xss::filter($testItem->vendor),
      ],
      [
      ['data' => $this->t('Product'), 'header' => TRUE],
        Xss::filter($testItem->product),
      ],
      [
      ['data' => $this->t('Version'), 'header' => TRUE],
        Xss::filter($testItem->version),
      ],
      [
      ['data' => $this->t('Latest'), 'header' => TRUE],
        Xss::filter($testItem->latest),
      ],
      [
      ['data' => $this->t('Abandoned'), 'header' => TRUE],
        $testItem->abandoned == 1 ? "Yes" : "No",
      ],
      [
      ['data' => $this->t('Description'), 'header' => TRUE],
        Xss::filter($testItem->description),
      ],
      [
      ['data' => $this->t('Vulnerability'), 'header' => TRUE],
        Markup::create($this->getEmoji($testItem->if_vulnerability)),
      ],
    ];

    if ($testItem->warning != "") {
      $rows[] = [
        ['data' => $this->t('Warning'), 'header' => TRUE],
        Xss::filter($testItem->warning),
      ];
    }

    if ($testItem->if_vulnerability == 1 || $testItem->if_vulnerability == 3) {
      $vulnerabilityMarkup = "<h3>Listed Vulnerabilitites</h3>";
      $vulnerabilityInfo = $this->getDrupalClient->getVulnerabilities($testItem->vendor, strtolower($testItem->product));
      if (isset($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)])) {
        if (count($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)]) > 0) {
          foreach ($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)] as $item) {
            $advisoryId = Xss::filter($item['advisoryId']);
            $vulnerabilityMarkup .= "<p><b>advisoryId:</b> {$advisoryId}</p>";
            $packageName = Xss::filter($item['packageName']);
            $vulnerabilityMarkup .= "<p><b>packageName:</b> {$packageName}</p>";
            $title = Xss::filter($item['title']);
            $vulnerabilityMarkup .= "<p><b>title:</b> {$title}</p>";
            $link = Link::fromTextAndUrl(
              $item['link'],
              Url::fromUri($item['link'])
            )->toString();
            $vulnerabilityMarkup .= $link;
            $affectedVersions = Xss::filter($item['affectedVersions']);
            $vulnerabilityMarkup .= "<p><b>affectedVersions:</b> {$affectedVersions}</p>";
            $reportedAt = Xss::filter($item['reportedAt']);
            $vulnerabilityMarkup .= "<p><b>reportedAt:</b> {$reportedAt}</p>";
            $composerRepository = UrlHelper::filterBadProtocol($item['composerRepository']);
            $vulnerabilityMarkup .= "<p><b>composerRepository:</b> {$composerRepository}</p>";
          }
        }
      }

      $rows[] = [
        ['data' => $this->t('Vulnerability Report'), 'header' => TRUE],
        Markup::create($vulnerabilityMarkup),
      ];
    }

    $build['vulnerability_test_item_table'] = [
      '#type' => 'table',
      '#rows' => $rows,
    ];

    return $build;
  }

  /**
   * Displays details about a specific database log message.
   *
   * @param int $event_id
   *   Unique ID of the database log message.
   *
   * @return array
   *   If the ID is located in the Database Logging table, a build array in the
   *   format expected by \Drupal\Core\Render\RendererInterface::render().
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   If no event found for the given ID.
   */
  public function libraryEventDetails($event_id) {
    $testItem = $this->database->query('SELECT [lpml].* FROM {lp_module_library} [lpml] WHERE [lpml].[id] = :id', [':id' => $event_id])->fetchObject();

    if (empty($testItem)) {
      throw new NotFoundHttpException();
    }

    $build = [];
    $rows = [
      [
      ['data' => $this->t('Date Created'), 'header' => TRUE],
        $this->dateFormatter->format($testItem->created, 'long'),
      ],
      [
        ['data' => $this->t('Name'), 'header' => TRUE],
        Markup::create(Xss::filterAdmin($testItem->name)),
      ],
      [
        ['data' => $this->t('Path'), 'header' => TRUE],
        UrlHelper::filterBadProtocol(urldecode($testItem->path)),
      ],
      [
      ['data' => $this->t('Type'), 'header' => TRUE],
        $testItem->type,
      ],
      [
      ['data' => $this->t('Vendor'), 'header' => TRUE],
        Xss::filter($testItem->vendor),
      ],
      [
      ['data' => $this->t('Product'), 'header' => TRUE],
        Xss::filter($testItem->product),
      ],
      [
      ['data' => $this->t('Version'), 'header' => TRUE],
        Xss::filter($testItem->version),
      ],
      [
      ['data' => $this->t('Latest'), 'header' => TRUE],
        Xss::filter($testItem->latest),
      ],
      [
      ['data' => $this->t('Abandoned'), 'header' => TRUE],
        $testItem->abandoned == 1 ? "Yes" : "No",
      ],
      [
      ['data' => $this->t('Description'), 'header' => TRUE],
        Markup::create(Xss::filterAdmin($testItem->description)),
      ],
      [
      ['data' => $this->t('Vulnerability'), 'header' => TRUE],
        Markup::create($this->getEmoji($testItem->if_vulnerability)),
      ],
    ];

    if ($testItem->warning != "") {
      $rows[] = [
        ['data' => $this->t('Warning'), 'header' => TRUE],
        Xss::filter($testItem->warning),
      ];
    }

    if ($testItem->if_vulnerability == 0 || $testItem->if_vulnerability == 1) {
      $vulnerabilityMarkup = "<h3>Listed Vulnerabilitites</h3>";
      $vulnerabilityInfo = $this->getDrupalClient->getVulnerabilities($testItem->vendor, strtolower($testItem->product));
      if (isset($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)])) {
        if (count($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)]) > 0) {
          foreach ($vulnerabilityInfo['advisories'][$testItem->vendor . "/" . strtolower($testItem->product)] as $item) {
            $advisoryId = Xss::filter($item['advisoryId']);
            $vulnerabilityMarkup .= "<p><b>advisoryId:</b> {$advisoryId}</p>";
            $packageName = Xss::filter($item['packageName']);
            $vulnerabilityMarkup .= "<p><b>packageName:</b> {$packageName}</p>";
            $title = Xss::filter($item['title']);
            $vulnerabilityMarkup .= "<p><b>title:</b> {$title}</p>";
            $link = Link::fromTextAndUrl(
              $item['link'],
              Url::fromUri($item['link'])
            )->toString();
            $vulnerabilityMarkup .= $link;
            $affectedVersions = Xss::filter($item['affectedVersions']);
            $vulnerabilityMarkup .= "<p><b>affectedVersions:</b> {$affectedVersions}</p>";
            $reportedAt = Xss::filter($item['reportedAt']);
            $vulnerabilityMarkup .= "<p><b>reportedAt:</b> {$reportedAt}</p>";
            $composerRepository = UrlHelper::filterBadProtocol($item['composerRepository']);
            $vulnerabilityMarkup .= "<p><b>composerRepository:</b> {$composerRepository}</p>";
          }
        }
      }

      $rows[] = [
        ['data' => $this->t('Vulnerability Report'), 'header' => TRUE],
        Markup::create($vulnerabilityMarkup),
      ];
    }

    $build['vulnerability_test_item_table'] = [
      '#type' => 'table',
      '#rows' => $rows,
    ];

    return $build;
  }

  /**
   * Builds a query for database log administration filters based on session.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array|null
   *   An associative array with keys 'where' and 'args' or NULL if there were
   *   no filters set.
   */
  protected function buildModuleLibrariesFilterQuery(Request $request) {
    $session_filters = $request->getSession()->get('list_module_libraries_overview_filter', []);
    // dd($session_filters);
    if (empty($session_filters)) {
      return;
    }

    $this->moduleHandler()->loadInclude('list_packages', 'admin.inc');

    $filters = list_module_libraries_filters();

    // dd($filters);
    // Build query.
    $where = $args = [];
    foreach ($session_filters as $key => $filter) {
      if ($filter != "" && $filter != "") {
        $filter_where = [];
        $filter_where[] = $filters[$key]['where'];
        $args[] = $filter;

        if (!empty($filter_where)) {
          $where[] = '(' . implode(' OR ', $filter_where) . ')';
        }
      }
    }
    $where = !empty($where) ? implode(' AND ', $where) : '';

    return [
      'where' => $where,
      'args' => $args,
    ];
  }

  /**
   * Builds a query for database log administration filters based on session.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return array|null
   *   An associative array with keys 'where' and 'args' or NULL if there were
   *   no filters set.
   */
  protected function buildModulesFilterQuery(Request $request) {
    $session_filters = $request->getSession()->get('list_modules_overview_filter', []);
    // dd($session_filters);
    if (empty($session_filters)) {
      return;
    }

    $this->moduleHandler()->loadInclude('list_packages', 'admin.inc');

    $filters = list_modules_filters();

    // dd($filters);
    // Build query.
    $where = $args = [];
    foreach ($session_filters as $key => $filter) {
      if ($filter != "") {
        $filter_where = [];
        $filter_where[] = $filters[$key]['where'];
        $args[] = $filter;

        if (count($filter_where) > 0) {
          $where[] = '(' . implode(' OR ', $filter_where) . ')';
        }
      }
    }
    $where = !empty($where) ? implode(' AND ', $where) : '';

    return [
      'where' => $where,
      'args' => $args,
    ];
  }

}
