<?php

namespace Drupal\list_packages\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class ListModuleLibrariesFilterForm extends FormBase {

  /**
   * The array of filters.
   *
   * @var array
   */
  private $filters = [
    'type',
    'if_internal',
    'if_vulnerability',
    'name',
  ];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'list_module_libraries_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $filters = list_module_libraries_filters();

    $session_filters = $this->getRequest()->getSession()->get('list_module_libraries_overview_filter', []);

    $modulelibrariesLink = Link::createFromRoute('Module & Theme Report', 'list_packages.installed_modules')
      ->toString()
      ->getGeneratedLink();

    $description = 'To load modules and libraries go to ' . $modulelibrariesLink . '.';

    $form['filters'] = [
      '#type' => 'details',
      '#title' => $this->t('Filter log messages'),
      '#description' => $description,
      '#open' => TRUE,
      '#attached' => [
        'library' => [
          'testsuite/testsuite.filters',
        ],
      ],
    ];

    $form['filters']['filters-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-filter-form-wrapper',
      ],
    ];

    $ifInternalDropdown = $form_state->hasValue('if_internal') ? $form_state->getValue('if_internal') : '';
    $if_internal_dropdown = $session_filters['if_internal'] ?? $ifInternalDropdown;

    $form['filters']['filters-container']['if_internal-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-if-internal-container',
      ],
    ];
    $form['filters']['filters-container']['if-internal-container']['if_internal'] = [
      '#title' => 'If Internal',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        0 => $this->t('External'),
        1 => $this->t('Internal'),
      ],
      '#default_value' => $if_internal_dropdown,
    ];

    $ifVulnerabilityDropdown = $form_state->hasValue('if_vulnerability') ? $form_state->getValue('if_vulnerability') : '';
    $if_vulnerability_dropdown = $session_filters['if_vulnerability'] ?? $ifVulnerabilityDropdown;

    $form['filters']['filters-container']['if-vulnerabilities-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-if-vulnerabilities-container',
      ],
    ];
    $form['filters']['filters-container']['if-vulnerabilities-container']['if_vulnerability'] = [
      '#title' => 'If Vulnerability',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        0 => $this->t('Vulnerabilities'),
        1 => $this->t('Not Definative'),
        2 => $this->t('No Information'),
        3 => $this->t('No Vulnerabilities'),
        4 => $this->t('Internal Scripts'),
      ],
      '#default_value' => $if_vulnerability_dropdown,
    ];

    $typeDropdown = $form_state->hasValue('type') ? $form_state->getValue('type') : '';
    $type_dropdown = $session_filters['type'] ?? $typeDropdown;

    $form['filters']['filters-container']['type-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-types-container',
      ],
    ];
    $form['filters']['filters-container']['type-container']['type'] = [
      '#title' => 'Type',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        'css' => $this->t('Css'),
        'js' => $this->t('Js'),
      ],
      '#default_value' => $type_dropdown,
    ];

    $nameDropdown = $form_state->hasValue('name') ? $form_state->getValue('name') : '';
    $name_dropdown = !empty($session_filters['name']) ? $session_filters['name'] : $nameDropdown;

    $form['filters']['filters-container']['name-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-name-container',
      ],
    ];
    $form['filters']['filters-container']['name-container']['name'] = [
      '#title' => 'Module or Theme Name',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => $filters['name']['options'] ?? "",
      '#default_value' => $name_dropdown,
    ];

    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
    if (!empty($session_filters)) {
      $form['filters']['actions']['reset'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset'),
        '#limit_validation_errors' => [],
        '#submit' => ['::resetForm'],
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!preg_match('/[0-9]{1}|/', $form_state->getValue('if_internal'))) {
      $form_state->setErrorByName('if_internal', $this->t('Invalid option.'));
    }
    if (!preg_match('/[0-9]{1}|/', $form_state->getValue('if_vulnerability'))) {
      $form_state->setErrorByName('if_vulnerability', $this->t('Invalid option.'));
    }
    if (!in_array($form_state->getValue('type'), ['', 'css', 'js'])) {
      $form_state->setErrorByName('type', $this->t('Invalid option.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $session_filters = $this->getRequest()->getSession()->get('list_module_libraries_overview_filter', []);

    foreach ($this->filters as $name) {
      if ($form_state->hasValue($name)) {
        $session_filters[$name] = $form_state->getValue($name);
      }
    }
    $this->getRequest()->getSession()->set('list_module_libraries_overview_filter', $session_filters);
  }

  /**
   * Resets the filter form.
   */
  public function resetForm() {
    $this->getRequest()->getSession()->remove('list_module_libraries_overview_filter');
  }

}
