<?php

namespace Drupal\list_packages\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class ListModulesFilterForm extends FormBase {

  /**
   * The array of filters.
   *
   * @var array
   */
  private $filters = [
    'type',
    'if_vulnerability',
  ];

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * ListModulesFilterForm constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   */
  public function __construct(
    Connection $connection,
  ) {
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
        $container->get('database'),
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'list_modules_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $session_filters = $this->getRequest()->getSession()->get('list_modules_overview_filter', []);

    $librariesLink = Link::createFromRoute('Module & Theme Library Report', 'list_packages.installed_module_libraries')
      ->toString()
      ->getGeneratedLink();

    $description = 'To view module and theme libraries go to ' . $librariesLink . '.';

    $form['filters'] = [
      '#type' => 'details',
      '#title' => $this->t('Filter log messages'),
      '#description' => $description,
      '#open' => TRUE,
      '#attached' => [
        'library' => [
          'testsuite/testsuite.filters',
        ],
      ],
    ];

    $form['filters']['filters-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-filter-form-wrapper',
      ],
    ];

    $ifVulnerabilityDropdown = $form_state->hasValue('if_vulnerability') ? $form_state->getValue('if_vulnerability') : '';
    $if_vulnerability_dropdown = $session_filters['if_vulnerability'] ?? $ifVulnerabilityDropdown;

    $form['filters']['filters-container']['if-vulnerabilities-container'] = [
      '#type' => 'container',
    ];
    $form['filters']['filters-container']['if-vulnerabilities-container']['if_vulnerability'] = [
      '#title' => 'If Vulnerability',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        0 => $this->t('Vulnerabilities'),
        1 => $this->t('Not Definative'),
        2 => $this->t('No Information'),
        3 => $this->t('No Vulnerabilities'),
        4 => $this->t('Internal Scripts'),
      ],
      '#default_value' => $if_vulnerability_dropdown,
    ];

    $typeDropdown = $form_state->hasValue('type') ? $form_state->getValue('type') : '';
    $type_dropdown = !empty($session_filters['type']) ? $session_filters['type'] : $typeDropdown;

    $form['filters']['filters-container']['type-container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'testsuite-tests-types-container',
      ],
    ];
    $form['filters']['filters-container']['type-container']['type'] = [
      '#title' => 'Modules & Themes',
      '#type' => 'select',
      '#multiple' => FALSE,
      '#size' => 8,
      '#options' => [
        'core-module' => $this->t('Core Module'),
        'contrib-module' => $this->t('Contrib Module'),
        'custom-module' => $this->t('Custom Module'),
        'core-theme' => $this->t('Core Theme'),
        'contrib-theme' => $this->t('Contrib Theme'),
        'custom-theme' => $this->t('Custom Theme'),
      ],
      '#default_value' => $type_dropdown,
    ];

    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
    if (!empty($session_filters)) {
      $form['filters']['actions']['reset'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset'),
        '#limit_validation_errors' => [],
        '#submit' => ['::resetForm'],
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!preg_match('/[0-9]{1}|/', $form_state->getValue('if_vulnerability'))) {
      $form_state->setErrorByName('if_vulnerability', $this->t('Invalid option.'));
    }
    if (!in_array($form_state->getValue('type'), [
      '',
      'core-module',
      'contrib-module',
      'custom-module',
      'core-theme',
      'contrib-theme',
      'custom-theme',
    ])) {
      $form_state->setErrorByName('type', $this->t('Invalid option.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $session_filters = $this->getRequest()->getSession()->get('list_modules_overview_filter', []);

    foreach ($this->filters as $name) {
      if ($form_state->hasValue($name)) {
        $session_filters[$name] = $form_state->getValue($name);
      }
    }
    $this->getRequest()->getSession()->set('list_modules_overview_filter', $session_filters);
  }

  /**
   * Resets the filter form.
   */
  public function resetForm() {
    $this->getRequest()->getSession()->remove('list_modules_overview_filter');
  }

}
