<?php

namespace Drupal\list_packages\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\list_packages\ListPackagesRepositoryService;
use Drupal\list_packages\ListPackagesResourceService;
use Drupal\testsuite\BaseTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class LoadCoreLibrariesForm extends FormBase {
  use BaseTrait;

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeInterface;

  /**
   * The Core, Modules and Themes libraries service.
   *
   * @var \Drupal\list_packages\ListPackagesResourceService
   */
  protected $listPackagesResourceService;

  /**
   * The repository service.
   *
   * @var \Drupal\list_packages\ListPackagesRepositoryService
   */
  protected $listPackagesRepositoryService;

  /**
   * LoadCoreLibrariesForm constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Load messenger service.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The time interface.
   * @param \Drupal\list_packages\ListPackagesResourceService $listPackagesResourceService
   *   The vulnerability resource service.
   * @param \Drupal\list_packages\ListPackagesRepositoryService $listPackagesRepositoryService
   *   The vulnerability resource service.
   */
  public function __construct(
    Connection $connection,
    MessengerInterface $messenger,
    TimeInterface $timeInterface,
    ListPackagesResourceService $listPackagesResourceService,
    ListPackagesRepositoryService $listPackagesRepositoryService,
  ) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->timeInterface = $timeInterface;
    $this->listPackagesResourceService = $listPackagesResourceService;
    $this->listPackagesRepositoryService = $listPackagesRepositoryService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('datetime.time'),
      $container->get('list_packages.load_resource.service'),
      $container->get('list_packages.repository.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'load_core_libraries_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#attributes' => ['class' => ['button', 'button--primary']],
      '#value' => $this->t('Load Core Libraries'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->connection->truncate('lp_core_library')->execute();
    $coreLibraries = $this->listPackagesResourceService->getCoreLibraries();
    $ifDbError = FALSE;
    $internalCssCount = 0;
    $externalCssCount = 0;
    $internalJsCount = 0;
    $externalJsCount = 0;
    $cssVulnerabilityCount = 0;
    $cssNoInformationCount = 0;
    $jsVulnerabilityCount = 0;
    $jsNoInformationCount = 0;
    $externalCssOriginList = [];
    $externalJsOriginList = [];
    $vunerabilityArray = [];
    $if_vulnerability = 2;
    foreach ($coreLibraries as $name => $mod) {
      if (isset($mod['packagist']['status']) && $mod['packagist']['status']) {
        $if_vulnerability = $mod['packagist']['status']['if_vulnerability'];
      }
      if (isset($mod['npm']['status']) && $mod['npm']['status']) {
        $if_vulnerability = $mod['npm']['status']['if_vulnerability'];
      }
      $ifCssDbError = FALSE;
      foreach ($mod['css'] as $libraryInfo) {
        if (isset($mod['remote']) && preg_match($this->regex['http_check'], $mod['remote'])) {
          $libName = $name;
        }
        else {
          $parts = explode('/', $libraryInfo['data']);
          $libName = end($parts);
        }
        if (preg_match('/^https?/', $libraryInfo['data'])) {
          $origin = parse_url($libraryInfo['data'], PHP_URL_HOST);
          if (!in_array($origin, $externalJsOriginList)) {
            $externalJsOriginList[] = $origin;
          }
        }

        $cssResult = $this->saveToDatabase('css', $name, $libName, $mod, $if_vulnerability, $libraryInfo['internal']);
        if (!$cssResult) {
          $ifCssDbError = TRUE;
        }
        else {
          if ($libraryInfo['internal']) {
            $internalCssCount++;
          }
          else {
            $externalCssCount++;
            if ($if_vulnerability >= 0 && $if_vulnerability <= 2) {
              $vunerabilityArray[] = [
                'table' => 0,
                'foreign_key_id' => $this->connection->lastInsertId(),
                'created' => $this->timeInterface->getRequestTime(),
              ];
            }
            if ($if_vulnerability == 0 || $if_vulnerability == 1) {
              $cssVulnerabilityCount++;
            }
            if ($if_vulnerability == 2) {
              $cssNoInformationCount++;
            }
          }
        }
        if ($ifCssDbError) {
          $this->messenger->addStatus('Installed library for ' . $libName . ' could not be recorded. Database error.');
        }
      }
      $ifJsDbError = FALSE;
      foreach ($mod['js'] as $libraryInfo) {
        if (preg_match($this->regex['http_check'], $libraryInfo['data'])) {
          $libName = $name;
        }
        else {
          $parts = explode('/', $libraryInfo['data']);
          $libName = end($parts);
        }
        if (preg_match('/^https?/', $libraryInfo['data'])) {
          $origin = parse_url($libraryInfo['data'], PHP_URL_HOST);
          if (!in_array($origin, $externalJsOriginList)) {
            $externalJsOriginList[] = $origin;
          }
        }

        if (preg_match('/^polyfill/', $libName)) {
          $libName = urldecode($libName);
          $parts = explode("=", $libName);
          $typeParts = explode(",", end($parts));
          $libName = "polyfill.min.js features:<br />";
          if (count($typeParts)) {
            for ($i = 0; $i <= count($typeParts); $i += 2) {
              $second = isset($typeParts[$i + 1]) ? ", " . $typeParts[$i + 1] : "";
              $third = isset($typeParts[$i + 2]) ? ", " . $typeParts[$i + 2] : "";
              $libName .= $typeParts[$i] . $second . $third . "<br />";
            }
          }
        }

        $jsResult = $this->saveToDatabase('js', $name, $libName, $mod, $if_vulnerability, $libraryInfo['internal']);
        if (!$jsResult) {
          $ifJsDbError = TRUE;
        }
        else {
          if ($libraryInfo['internal']) {
            $internalJsCount++;
          }
          else {
            $externalJsCount++;
            if ($if_vulnerability >= 0 && $if_vulnerability <= 2) {
              $vunerabilityArray[] = [
                'table' => 0,
                'foreign_key_id' => $this->connection->lastInsertId(),
                'created' => $this->timeInterface->getRequestTime(),
              ];
            }
            if ($if_vulnerability == 0 || $if_vulnerability == 1) {
              $jsVulnerabilityCount++;
            }
            if ($if_vulnerability == 2) {
              $jsNoInformationCount++;
            }
          }
        }
        if ($ifJsDbError) {
          $this->messenger->addStatus('Installed library for ' . $libName . ' could not be recorded. Database error.');
        }
      }
    }
    if ($ifDbError) {
      $this->messenger->addStatus('Installed packages could not be recorded. Database error.');
    }
    else {
      $cssReportString = '';
      if ($internalCssCount > 0) {
        $cssReportString .= '<span class="testsuite-blue"><b>' . $internalCssCount . " are Internal Css Scripts</b></span><br />";
      }
      $cssReportString .= $externalCssCount . ' external css libraries.<br />';
      if ($cssNoInformationCount > 0) {
        $cssReportString .= '<span class="testsuite-red"><b>' . $cssNoInformationCount . " have No Information</b></span><br />";
      }
      if ($cssNoInformationCount == 0 && $cssVulnerabilityCount == 0) {
        $cssReportString .= '<span class="testsuite-green"><b>' . $cssVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($cssVulnerabilityCount > 0) {
        $cssReportString .= '<span class="testsuite-red"><b>' . $cssVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if (count($externalCssOriginList) > 0) {
        $cssReportString .= "External Css are using: <br />";
        foreach ($externalCssOriginList as $extCssOrigin) {
          $cssReportString .= "{$extCssOrigin}<br />";
        }
      }

      $jsReportString = '';
      if ($internalJsCount > 0) {
        $jsReportString .= '<span class="testsuite-blue"><b>' . $internalJsCount . " are Internal Js Scripts</b></span><br />";
      }
      $jsReportString .= $externalJsCount . ' external js libraries.<br />';
      if ($jsNoInformationCount > 0) {
        $jsReportString .= '<span class="testsuite-red"><b>' . $jsNoInformationCount . " have No Information</b></span><br />";
      }
      if ($jsNoInformationCount == 0 && $jsVulnerabilityCount == 0) {
        $jsReportString .= '<span class="testsuite-green"><b>' . $jsVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($jsVulnerabilityCount > 0) {
        $jsReportString .= '<span class="testsuite-red"><b>' . $jsVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if (count($externalJsOriginList) > 0) {
        $jsReportString .= "External Js are using: <br />";
        foreach ($externalJsOriginList as $extJsOrigin) {
          $jsReportString .= "{$extJsOrigin}<br />";
        }
      }

      $coreLibraryReport = $internalCssCount + $externalCssCount + $internalJsCount + $externalJsCount . ' core libraries have been recorded.<br />' . $cssReportString . $jsReportString;
      $this->messenger->addStatus($internalCssCount + $externalCssCount + $internalJsCount + $externalJsCount . ' core libraries have been recorded. ' . $internalCssCount . ' internal css libraries and ' . $externalCssCount . ' external css libraries. ' . $cssVulnerabilityCount . ' have vulnerabilities. ' . $internalJsCount . ' internal js libraries and ' . $externalJsCount . ' external js libraries. ' . $jsVulnerabilityCount . ' have vulnerabilities. ');
      $this->listPackagesRepositoryService->createDashboardLog(0, [
        'core_int_css_libraries' => $internalCssCount,
        'core_int_js_libraries' => $internalJsCount,
        'core_ext_css_libraries' => $externalCssCount,
        'core_ext_js_libraries' => $externalJsCount,
        'core_library_vulnerabilities' => $cssVulnerabilityCount + $jsVulnerabilityCount,
        'core_library_report' => $coreLibraryReport,
      ]);
      $this->listPackagesRepositoryService->createVulnerabilityLogs(0, $vunerabilityArray);
    }
  }

  /**
   * Sanitizes and saves data to the database.
   *
   * @param string $type
   *   Js or Css.
   * @param string $name
   *   The initial name before exploding into vendor and product.
   * @param string $libName
   *   The product of the exploded name.
   * @param array $mod
   *   The information on the library.
   * @param int $if_vulnerability
   *   The vulnerability result.
   * @param int $ifInternal
   *   If the script is an internal or external script.
   *
   * @return \Drupal\Core\Database\Query\Insert
   *   A new Insert query object.
   */
  private function saveToDatabase($type, $name, $libName, $mod, $if_vulnerability, $ifInternal) {
    $name = preg_match($this->regex['name_dot_name'], $name) ? $name : "";
    if ($name) {
      $parts = explode(".", $name);
      $mod['vendor'] = $parts[0] ?? $name;
      $mod['product'] = $parts[1] ?? $name;
    }
    if ($type == 'js') {
      $libName = preg_match($this->regex['js_file_name'], $libName) ? $libName : "";
    }
    else {
      $libName = preg_match($this->regex['css_file_name'], $libName) ? $libName : "";
    }
    $mod['extension_name'] = preg_match($this->regex['name_dot_name'], $mod['extension_name']) ? $mod['extension_name'] : "";
    $if_vulnerability = preg_match($this->regex['numbers'], $if_vulnerability) ? $if_vulnerability : "";
    if (isset($mod['vendor'])) {
      $mod['vendor'] = preg_match($this->regex['string'], $mod['vendor']) ? $mod['vendor'] : "";
    }
    if (isset($mod['product'])) {
      $mod['product'] = preg_match($this->regex['string'], $mod['product']) ? $mod['product'] : "";
    }
    if (isset($mod['version'])) {
      $mod['version'] = preg_match($this->regex['semver_version'], $mod['version']) ? $mod['version'] : "";
    }

    return $this->connection->insert('lp_core_library')
      ->fields(
      [
        'type' => $type,
        'area' => 'core',
        'name' => $libName,
        'description' => "{$libName} file.",
        'if_internal' => $ifInternal ? 1 : 0,
        'if_vulnerability' => $ifInternal ? 4 : $if_vulnerability,
        'warning' => "",
        'vendor' => $mod['vendor'] ?? $name,
        'product' => $mod['product'] ?? $name,
        'version' => $mod['version'] ?? "ANY",
        'latest' => $mod['version'] ?? "ANY",
        'latest_status' => "ANY",
        'abandoned' => 0,
        'created' => $this->timeInterface->getRequestTime(),
      ]
    )
      ->execute();
  }

}
