<?php

namespace Drupal\list_packages\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\list_packages\ListPackagesRepositoryService;
use Drupal\list_packages\ListPackagesResourceService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class LoadLibrariesForm extends FormBase {

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeInterface;

  /**
   * The Core, Modules and Themes libraries service.
   *
   * @var \Drupal\list_packages\ListPackagesResourceService
   */
  protected $listPackagesResourceService;

  /**
   * The repository service.
   *
   * @var \Drupal\list_packages\ListPackagesRepositoryService
   */
  protected $listPackagesRepositoryService;

  /**
   * LoadLibrariesForm constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Load messenger service.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The time interface.
   * @param \Drupal\list_packages\ListPackagesResourceService $listPackagesResourceService
   *   The vulnerability resource service.
   * @param \Drupal\list_packages\ListPackagesRepositoryService $listPackagesRepositoryService
   *   The vulnerability resource service.
   */
  public function __construct(
    Connection $connection,
    MessengerInterface $messenger,
    TimeInterface $timeInterface,
    ListPackagesResourceService $listPackagesResourceService,
    ListPackagesRepositoryService $listPackagesRepositoryService,
  ) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->timeInterface = $timeInterface;
    $this->listPackagesResourceService = $listPackagesResourceService;
    $this->listPackagesRepositoryService = $listPackagesRepositoryService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('datetime.time'),
      $container->get('list_packages.load_resource.service'),
      $container->get('list_packages.repository.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'load_libraries_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#attributes' => ['class' => ['button', 'button--primary']],
      '#value' => $this->t('Load Libraries'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->connection->truncate('lp_installed_library')->execute();
    $libraries = $this->listPackagesResourceService->getLibrariesPackages();
    if (count($libraries) > 0) {
      $ifDbError = FALSE;
      $count = 0;
      $vulnerabilityCount = 0;
      $noInformationCount = 0;
      $vunerabilityArray = [];
      $githubApiCallsFailed = $libraries['github_api_failed'];
      $githubApiCalls = $libraries['github_api_calls'];
      foreach ($libraries as $name => $line) {
        if ($name != "github_api_failed" && $name != "github_api_calls") {
          $result = $this->connection->insert('lp_installed_library')
            ->fields(
              [
                'path' => $line['path'],
                'type' => $line['type'],
                'area' => 'core',
                'name' => $line['name'] ? $line['name'] : "",
                'description' => $line['description'] ? $line['description'] : "",
                'if_vulnerability' => $line['if_vulnerability'],
                'warning' => $line['warning'] ? $line['warning'] : "",
                'vendor' => $line['vendor'] ? $line['vendor'] : "",
                'product' => $line['product'] ? $line['product'] : "",
                'version' => $line['version'] ? $line['version'] : "ANY",
                'latest' => $line['latest'] ? $line['latest'] : "ANY",
                'latest_status' => $line['latest-status'] ? $line['latest-status'] : "ANY",
                'abandoned' => $line['abandoned'] == TRUE ? 1 : 0,
                'created' => $this->timeInterface->getRequestTime(),
              ]
            )
            ->execute();
          if (!$result) {
            $ifDbError = TRUE;
          }
          else {
            $count++;
            if ($line['if_vulnerability'] >= 0 && $line['if_vulnerability'] <= 2) {
              $vunerabilityArray[] = [
                'table' => 2,
                'foreign_key_id' => $this->connection->lastInsertId(),
                'created' => $this->timeInterface->getRequestTime(),
              ];
            }
            if ($line['if_vulnerability'] == 0 || $line['if_vulnerability'] == 1) {
              $vulnerabilityCount++;
            }
            if ($line['if_vulnerability'] == 2) {
              $noInformationCount++;
            }
          }
        }
      }
      if ($ifDbError) {
        $this->messenger->addStatus('Installed libraries could not be recorded. Database error.');
      }
      else {
        $librariesReport = $count . ' Installed libraries have been recorded.<br />';
        if ($noInformationCount > 0) {
          $librariesReport .= '<span class="testsuite-red"><b>' . $noInformationCount . " have No Information</b></span><br />";
        }
        if ($noInformationCount == 0 && $vulnerabilityCount == 0) {
          $librariesReport .= '<span class="testsuite-green"><b>' . $vulnerabilityCount . " have Vulnerabilities</b></span><br />";
        }
        if ($vulnerabilityCount > 0) {
          $librariesReport .= '<span class="testsuite-red"><b>' . $vulnerabilityCount . " have Vulnerabilities</b></span><br />";
        }
        if ($githubApiCallsFailed) {
          $librariesReport .= '<span class="testsuite-red"><b>Github API rate limit reached.<br />Wait 1 hour and try again.</b></span><br />';
        }
        else {
          $librariesReport .= '<span class="testsuite-green"><b>Github API success: ' . $githubApiCalls . '</b></span><br />';
        }

        $this->messenger->addStatus($count . ' Installed libraries have been recorded. ' . $vulnerabilityCount . ' have vulnerabilities.');
        $this->listPackagesRepositoryService->createDashboardLog(2, [
          'libraries' => $count,
          'library_vulnerabilities' => $vulnerabilityCount,
          'libraries_report' => $librariesReport,
        ]);
        $this->listPackagesRepositoryService->createVulnerabilityLogs(2, $vunerabilityArray);
      }
    }
    else {
      $this->messenger->addStatus('You have no libraries installed.');
    }
  }

}
