<?php

namespace Drupal\list_packages\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\list_packages\ListPackagesRepositoryService;
use Drupal\list_packages\ListPackagesResourceService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the database logging filter form.
 *
 * @internal
 */
class LoadModsThemsForm extends FormBase {

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeInterface;

  /**
   * The Core, Modules and Themes libraries service.
   *
   * @var \Drupal\list_packages\ListPackagesResourceService
   */
  protected $listPackagesResourceService;

  /**
   * The repository service.
   *
   * @var \Drupal\list_packages\ListPackagesRepositoryService
   */
  protected $listPackagesRepositoryService;

  /**
   * LoadModsThemsForm constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Load messenger service.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The time interface.
   * @param \Drupal\list_packages\ListPackagesResourceService $listPackagesResourceService
   *   The vulnerability resource service.
   * @param \Drupal\list_packages\ListPackagesRepositoryService $listPackagesRepositoryService
   *   The vulnerability resource service.
   */
  public function __construct(
    Connection $connection,
    MessengerInterface $messenger,
    TimeInterface $timeInterface,
    ListPackagesResourceService $listPackagesResourceService,
    ListPackagesRepositoryService $listPackagesRepositoryService,
  ) {
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->timeInterface = $timeInterface;
    $this->listPackagesResourceService = $listPackagesResourceService;
    $this->listPackagesRepositoryService = $listPackagesRepositoryService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('messenger'),
      $container->get('datetime.time'),
      $container->get('list_packages.load_resource.service'),
      $container->get('list_packages.repository.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'load_mods_thems_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#attributes' => ['class' => ['container-inline']],
    ];
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#attributes' => ['class' => ['button', 'button--primary']],
      '#value' => $this->t('Load Modules & Themes'),
    ];

    return $form;
  }

  /**
   * If drupal module get area.
   *
   * @return string
   *   The area. core contrib custom.
   */
  private function getArea($path) {
    if (preg_match('/\\\contrib\\\/', $path)) {
      return 'contrib';
    }
    if (preg_match('/\\\custom\\\/', $path)) {
      return 'custom';
    }
    return 'core';
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->connection->truncate('lp_installed_module')->execute();
    $this->connection->truncate('lp_module_library')->execute();
    $modulesThemes = $this->listPackagesResourceService->getModulesThemesAndTheirLibraries();
    $ifDbError = FALSE;
    $coreModuleCount = 0;
    $contribModuleCount = 0;
    $customModuleCount = 0;
    $coreThemeCount = 0;
    $contribThemeCount = 0;
    $customThemeCount = 0;
    $internalCssCount = 0;
    $externalCssCount = 0;
    $internalJsCount = 0;
    $externalJsCount = 0;
    $modVulnerabilityCount = 0;
    $modNoInformationCount = 0;
    $themeVulnerabilityCount = 0;
    $themeNoInformationCount = 0;
    $cssVulnerabilityCount = 0;
    $cssNoInformationCount = 0;
    $jsVulnerabilityCount = 0;
    $jsNoInformationCount = 0;
    $modVulnerabilityArray = [];
    $modLibsVulnerabilityArray = [];
    $externalCssOriginList = [];
    $externalJsOriginList = [];
    $if_vulnerability = 2;
    $githubApiCallsFailed = $modulesThemes['github_api_failed'];
    $githubApiCalls = $modulesThemes['github_api_calls'];
    foreach ($modulesThemes as $type => $mods) {
      if ($type != 'github_api_calls' && $type != 'github_api_failed') {
        foreach ($mods as $name => $mod) {
          if ($type == 'module' || $type == 'theme') {
            $area = $this->getArea($mod['info']['path']);
            if (isset($mod['info']['version']) && $mod['info']['version'] == 'VERSION') {
              $version = \DRUPAL::VERSION;
            }
            elseif (isset($mod['info']['version'])) {
              $version = $mod['info']['version'];
            }
            else {
              $version = $mod['show-info']['version'] ?? "ANY";
            }
            $description = "";
            if (isset($mod['show-info']['description'])) {
              $description = $mod['show-info']['description'];
            }
            elseif (isset($mod['info']['description'])) {
              $description = $mod['info']['description'];
            }
            $latest = $mod['show-info']['version'] ?? $version;
            $result = $this->connection->insert('lp_installed_module')
              ->fields(
              [
                'type' => $area . "-" . $type,
                'area' => $area,
                'name' => $mod['info']['name'] ?? "No name",
                'description' => $description ? $description : "No description",
                'if_vulnerability' => $mod['info']['vulnerabilities'],
                'warning' => $mod['info']['warning'] ?? "",
                'vendor' => $type == 'module' ? 'drupal' : 'theme',
                'product' => $name,
                'version' => $version,
                'latest' => $latest,
                'latest_status' => $mod['show-info']['latest-status'] ?? "ANY",
                'abandoned' => 0,
                'created' => $this->timeInterface->getRequestTime(),
              ]
            )
              ->execute();
            if ($mod['info']['vulnerabilities'] >= 0 && $mod['info']['vulnerabilities'] <= 2) {
              $modVulnerabilityArray[] = [
                'table' => 3,
                'foreign_key_id' => $this->connection->lastInsertId(),
                'created' => $this->timeInterface->getRequestTime(),
              ];
            }
            if ($mod['info']['vulnerabilities'] == 0 || $mod['info']['vulnerabilities'] == 1) {
              if ($type == 'module') {
                $modVulnerabilityCount++;
              }
              else {
                $themeVulnerabilityCount++;
              }
            }
            if ($mod['info']['vulnerabilities'] == 2) {
              if ($type == 'module') {
                $modNoInformationCount++;
              }
              else {
                $themeNoInformationCount++;
              }
            }
            if (isset($mod['libraries']) && count($mod['libraries']) > 0) {
              foreach ($mod['libraries'] as $libraryName => $library) {
                if (isset($mod['packagist']['status']) && $mod['packagist']['status']) {
                  $if_vulnerability = $mod['packagist']['status']['if_vulnerability'];
                }
                if (isset($mod['npm']['status']) && $mod['npm']['status']) {
                  $if_vulnerability = $mod['npm']['status']['if_vulnerability'];
                }
                $ifCssDbError = FALSE;
                foreach ($library['css'] as $libraryInfo) {
                  /* if ($libraryName == "jquery.farbtastic") {
                  dd($libraryName, $libraryInfo);
                  } */
                  $if_vulnerability = $libraryInfo["vulnerabilities"] ?? 2;
                  if (preg_match('/^https?/', $libraryInfo['data'])) {
                    $origin = parse_url($libraryInfo['data'], PHP_URL_HOST);
                    if (!in_array($origin, $externalCssOriginList)) {
                      $externalCssOriginList[] = $origin;
                    }
                  }
                  $parts = explode('/', $libraryInfo['data']);
                  $libName = end($parts);
                  $version = $libraryInfo['version'] == -1 ? '1.0' : $libraryInfo['version'];
                  $cssResult = $this->connection->insert('lp_module_library')
                    ->fields(
                  [
                    'lp_installed_module_id' => $result,
                    'type' => 'css',
                    'path' => $libraryInfo['path'],
                    'area' => $area,
                    'name' => $libName,
                    'description' => $libName,
                    'if_vulnerability' => $libraryInfo['internal'] ? 4 : $if_vulnerability,
                    'if_internal' => $libraryInfo['internal'] ? 1 : 0,
                    'warning' => "",
                    'vendor' => $libraryInfo['vendor'] ?? ($type == 'module' ? 'drupal' : 'theme'),
                    'product' => $libraryInfo['product'] ?? $libraryName,
                    'version' => $version,
                    'latest' => $version,
                    'latest_status' => "ANY",
                    'abandoned' => 0,
                    'created' => $this->timeInterface->getRequestTime(),
                  ]
                    )
                    ->execute();
                  if (!$cssResult) {
                    $ifCssDbError = TRUE;
                  }
                  else {
                    if ($libraryInfo['internal']) {
                      $internalCssCount++;
                    }
                    else {
                      $externalCssCount++;
                      if ($if_vulnerability >= 0 && $if_vulnerability <= 2) {
                        $modLibsVulnerabilityArray[] = [
                          'table' => 4,
                          'foreign_key_id' => $this->connection->lastInsertId(),
                          'created' => $this->timeInterface->getRequestTime(),
                        ];
                      }
                      if ($if_vulnerability == 0 || $if_vulnerability == 1) {
                        $cssVulnerabilityCount++;
                      }
                      if ($if_vulnerability == 2) {
                        $cssNoInformationCount++;
                      }
                    }
                  }
                  if ($ifCssDbError) {
                    $this->messenger->addStatus('Installed library for ' . $libraryName . ' could not be recorded. Database error.');
                  }
                }
                $ifJsDbError = FALSE;
                foreach ($library['js'] as $libraryInfo) {
                  if (preg_match('/^https?/', $libraryInfo['data'])) {
                    $origin = parse_url($libraryInfo['data'], PHP_URL_HOST);
                    if (!in_array($origin, $externalJsOriginList)) {
                      $externalJsOriginList[] = $origin;
                    }
                  }
                  $parts = explode('/', $libraryInfo['data']);
                  $libName = end($parts);
                  if (preg_match('/^polyfill/', $libName)) {
                    $libName = urldecode($libName);
                    $parts = explode("=", $libName);
                    $typeParts = explode(",", end($parts));
                    $libName = "polyfill.min.js features:<br />";
                    if (count($typeParts)) {
                      for ($i = 0; $i <= count($typeParts); $i += 2) {
                        $second = isset($typeParts[$i + 1]) ? ", " . $typeParts[$i + 1] : "";
                        $third = isset($typeParts[$i + 2]) ? ", " . $typeParts[$i + 2] : "";
                        $libName .= $typeParts[$i] . $second . $third . "<br />";
                      }
                    }
                  }
                  $if_vulnerability = $libraryInfo["vulnerabilities"] ?? 2;
                  $version = $libraryInfo['version'] == -1 ? '1.0' : $libraryInfo['version'];
                  $jsResult = $this->connection->insert('lp_module_library')
                    ->fields(
                  [
                    'lp_installed_module_id' => $result,
                    'type' => 'js',
                    'path' => $libraryInfo['path'],
                    'area' => $area,
                    'name' => $libName,
                    'description' => $libName,
                    'if_vulnerability' => $libraryInfo['internal'] ? 4 : $if_vulnerability,
                    'if_internal' => $libraryInfo['internal'] ? 1 : 0,
                    'warning' => "",
                    'vendor' => $libraryInfo['vendor'] ?? ($type == 'module' ? 'drupal' : 'theme'),
                    'product' => $libraryInfo['product'] ?? $libraryName,
                    'version' => $version,
                    'latest' => $version,
                    'latest_status' => "ANY",
                    'abandoned' => 0,
                    'created' => $this->timeInterface->getRequestTime(),
                  ]
                    )
                    ->execute();
                  if (!$jsResult) {
                    $ifJsDbError = TRUE;
                  }
                  else {
                    if ($libraryInfo['internal']) {
                      $internalJsCount++;
                    }
                    else {
                      $externalJsCount++;
                      if ($if_vulnerability >= 0 && $if_vulnerability <= 2) {
                        $modLibsVulnerabilityArray[] = [
                          'table' => 4,
                          'foreign_key_id' => $this->connection->lastInsertId(),
                          'created' => $this->timeInterface->getRequestTime(),
                        ];
                      }
                      if ($if_vulnerability == 0 || $if_vulnerability == 1) {
                        $jsVulnerabilityCount++;
                      }
                      if ($if_vulnerability == 2) {
                        $jsNoInformationCount++;
                      }
                    }
                  }
                  if ($ifJsDbError) {
                    $this->messenger->addStatus('Installed library for ' . $libraryName . ' could not be recorded. Database error.');
                  }
                }
              }
              if (!$result) {
                $ifDbError = TRUE;
              }
              else {
                if ($type == 'module') {
                  switch ($area) {
                    case 'core':
                      $coreModuleCount++;
                      break;

                    case 'contrib':
                      $contribModuleCount++;
                      break;

                    case 'custom':
                      $customModuleCount++;
                      break;
                  }
                }
                if ($type == 'theme') {
                  switch ($area) {
                    case 'core':
                      $coreThemeCount++;
                      break;

                    case 'contrib':
                      $contribThemeCount++;
                      break;

                    case 'custom':
                      $customThemeCount++;
                      break;
                  }
                }
              }
            }
          }
        }
      }
    }
    if ($ifDbError) {
      $this->messenger->addStatus('Installed packages could not be recorded. Database error.');
    }
    else {
      // dd($externalCssOriginList, $externalJsOriginList);.
      $modules = $coreModuleCount + $contribModuleCount + $customModuleCount;
      $themes = $coreThemeCount + $contribThemeCount + $customThemeCount;
      $modReportString = $modules . ' modules.<br />';
      if ($modNoInformationCount > 0) {
        $modReportString .= '<span class="testsuite-red"><b>' . $modNoInformationCount . " have No Information</b></span><br />";
      }
      if ($modNoInformationCount == 0 && $modVulnerabilityCount == 0) {
        $modReportString .= '<span class="testsuite-green"><b>' . $modVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($modVulnerabilityCount > 0) {
        $modReportString .= '<span class="testsuite-red"><b>' . $modVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      $themeReportString = $themes . ' themes.<br />';
      if ($themeNoInformationCount > 0) {
        $themeReportString .= '<span class="testsuite-red"><b>' . $themeNoInformationCount . " have No Information</b></span><br />";
      }
      if ($themeNoInformationCount == 0 && $themeVulnerabilityCount == 0) {
        $themeReportString .= '<span class="testsuite-green"><b>' . $themeVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($themeVulnerabilityCount > 0) {
        $themeReportString .= '<span class="testsuite-red"><b>' . $themeVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($githubApiCallsFailed) {
        $themeReportString .= '<span class="testsuite-red"><b>Github API rate limit reached.<br />Wait 1 hour and try again.</b></span><br />';
      }
      else {
        $themeReportString .= '<span class="testsuite-green"><b>Github API success: ' . $githubApiCalls . '</b></span><br />';
      }

      $cssReportString = '';
      if ($internalCssCount > 0) {
        $cssReportString .= '<span class="testsuite-blue"><b>' . $internalCssCount . " are Internal Css Scripts</b></span><br />";
      }
      $cssReportString .= $externalCssCount . ' external css libraries.<br />';
      if ($cssNoInformationCount > 0) {
        $cssReportString .= '<span class="testsuite-red"><b>' . $cssNoInformationCount . " have No Information</b></span><br />";
      }
      if ($cssNoInformationCount == 0 && $cssVulnerabilityCount == 0) {
        $cssReportString .= '<span class="testsuite-green"><b>' . $cssVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($cssVulnerabilityCount > 0) {
        $cssReportString .= '<span class="testsuite-red"><b>' . $cssVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if (count($externalCssOriginList) > 0) {
        $cssReportString .= "External Css are using: <br />";
        foreach ($externalCssOriginList as $extCssOrigin) {
          $cssReportString .= "{$extCssOrigin}<br />";
        }
      }

      $jsReportString = '';
      if ($internalJsCount > 0) {
        $jsReportString .= '<span class="testsuite-blue"><b>' . $internalJsCount . " are Internal Js Scripts</b></span><br />";
      }
      $jsReportString .= $externalJsCount . ' external js libraries.<br />';
      if ($jsNoInformationCount > 0) {
        $jsReportString .= '<span class="testsuite-red"><b>' . $jsNoInformationCount . " have No Information</b></span><br />";
      }
      if ($jsNoInformationCount == 0 && $jsVulnerabilityCount == 0) {
        $jsReportString .= '<span class="testsuite-green"><b>' . $jsVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if ($jsVulnerabilityCount > 0) {
        $jsReportString .= '<span class="testsuite-red"><b>' . $jsVulnerabilityCount . " have Vulnerabilities</b></span><br />";
      }
      if (count($externalJsOriginList) > 0) {
        $jsReportString .= "External Js are using: <br />";
        foreach ($externalJsOriginList as $extJsOrigin) {
          $jsReportString .= "{$extJsOrigin}<br />";
        }
      }

      $this->messenger->addStatus($modules + $themes . ' installed packages have been recorded. ' . $modules . ' modules and ' . $themes . ' themes. ' . $internalCssCount + $externalCssCount + $internalJsCount + $externalJsCount . ' libraries have been recorded. ' . $internalCssCount . ' internal css libraries. ' . $externalCssCount . ' external css libraries. ' . $internalJsCount . ' internal js libraries and ' . $externalJsCount . ' external js libraries. ' . $modVulnerabilityCount + $cssVulnerabilityCount + $jsVulnerabilityCount . ' have vulnerabilities.');
      $this->listPackagesRepositoryService->createDashboardLog(3, [
        'modules_report' => $modReportString,
        'themes_report' => $themeReportString,
      ]);
      $this->listPackagesRepositoryService->createDashboardLog(4, [
        'modsthems_ext_css_libraries_report' => $cssReportString,
        'modsthems_ext_js_libraries_report' => $jsReportString,
      ]);
      $this->listPackagesRepositoryService->createVulnerabilityLogs(3, $modVulnerabilityArray);
      $this->listPackagesRepositoryService->createVulnerabilityLogs(4, $modLibsVulnerabilityArray);
    }
  }

}
