<?php

namespace Drupal\list_packages;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;

/**
 * Provides the database repository.
 *
 * @internal
 */
class ListPackagesRepositoryService {

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeInterface;

  /**
   * ListPackagesRepositoryService constructor.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database.
   * @param \Drupal\Component\Datetime\TimeInterface $timeInterface
   *   The time interface.
   */
  public function __construct(
    Connection $connection,
    TimeInterface $timeInterface,
  ) {
    $this->connection = $connection;
    $this->timeInterface = $timeInterface;
  }

  /**
   * Creates aa combination of the data in the database and the incoming update.
   *
   * @param string $type
   *   The type of update.
   *   0 = core libraries
   *   1 = packages.
   *   2 = libraries.
   *   3 = modules & themes.
   *   4 = modules & themes libraries.
   * @param string $vars
   *   The array of database variables.
   *
   * @return bool
   *   The aggregated results.
   */
  private function aggregateUpdate($type, $vars) {
    $aggregated = [];
    $result = $this->connection->select('lp_dashboard', 'lpd')
      ->fields('lpd', [
        'core_library_report',
        'packages_report',
        'libraries_report',
        'modules_report',
        'themes_report',
        'modsthems_ext_css_libraries_report',
        'modsthems_ext_js_libraries_report',
      ])
      ->execute()->fetchAssoc();

    // dd($type);
    $aggregated['core_library_report'] = $result['core_library_report'];
    $aggregated['packages_report'] = $result['packages_report'];
    $aggregated['libraries_report'] = $result['libraries_report'];
    $aggregated['modules_report'] = $result['modules_report'];
    $aggregated['themes_report'] = $result['themes_report'];
    $aggregated['modsthems_ext_css_libraries_report'] = $result['modsthems_ext_css_libraries_report'];
    $aggregated['modsthems_ext_js_libraries_report'] = $result['modsthems_ext_js_libraries_report'];

    switch ($type) {
      case 0:
        $aggregated['core_library_report'] = $vars['core_library_report'];
        break;

      case 1:
        $aggregated['packages_report'] = $vars['packages_report'];
        break;

      case 2:
        $aggregated['libraries_report'] = $vars['libraries_report'];
        break;

      case 3:
        $aggregated['modules_report'] = $vars['modules_report'];
        $aggregated['themes_report'] = $vars['themes_report'];
        break;

      case 4:
        $aggregated['modsthems_ext_css_libraries_report'] = $vars['modsthems_ext_css_libraries_report'];
        $aggregated['modsthems_ext_js_libraries_report'] = $vars['modsthems_ext_js_libraries_report'];
        break;
    }

    return $aggregated;
  }

  /**
   * Creates a log entry.
   *
   * @param string $type
   *   The type of update.
   *   0 = core libraries
   *   1 = packages.
   *   2 = libraries.
   *   3 = modules & themes.
   *   4 = modules & themes libraries.
   * @param string $vars
   *   The array of database variables.
   *
   * @return bool
   *   True or false depending on if insert succeeds.
   */
  public function createDashboardLog($type, $vars) {
    $query = $this->connection->select('lp_dashboard', 'lpd');
    $query->addField('lpd', 'id');
    $results = $query->execute()->fetchAll();
    $count = count($results);
    // dd($results);
    if ($count > 0) {
      $vars = $this->aggregateUpdate($type, $vars);
      return $this->connection->update('lp_dashboard')
        ->fields(
        [
          'core_library_report' => $vars['core_library_report'],
          'packages_report' => $vars['packages_report'],
          'libraries_report' => $vars['libraries_report'],
          'modules_report' => $vars['modules_report'],
          'themes_report' => $vars['themes_report'],
          'modsthems_ext_css_libraries_report' => $vars['modsthems_ext_css_libraries_report'],
          'modsthems_ext_js_libraries_report' => $vars['modsthems_ext_js_libraries_report'],
        ]
      )
        ->condition('id', 1)
        ->execute();
    }
    else {
      return $this->connection->insert('lp_dashboard')
        ->fields(
        [
          'id' => 1,
          'core_library_report' => $vars['core_library_report'] ?? "",
          'packages_report' => $vars['packages_report'] ?? "",
          'libraries_report' => $vars['libraries_report'] ?? "",
          'modules_report' => $vars['modules_report'] ?? "",
          'themes_report' => $vars['themes_report'] ?? "",
          'modsthems_ext_css_libraries_report' => $vars['modsthems_ext_css_libraries_report'] ?? "",
          'modsthems_ext_js_libraries_report' => $vars['modsthems_ext_js_libraries_report'] ?? "",
          'created' => $this->timeInterface->getRequestTime(),
        ]
      )
        ->execute();
    }
  }

  /**
   * Creates a log entry.
   *
   * @param string $type
   *   The type of update.
   *   0 = core libraries
   *   1 = packages.
   *   2 = libraries.
   *   3 = modules & themes.
   *   4 = modules & themes libraries.
   * @param array $results
   *   The array of database variables.
   *
   * @return bool
   *   True or false depending on if insert succeeds.
   */
  public function createVulnerabilityLogs($type, $results) {
    $this->connection->delete('lp_vulnerabilities')
      ->condition('table', $type)
      ->execute();

    $ifError = FALSE;
    foreach ($results as $item) {
      $result = $this->connection->insert('lp_vulnerabilities')
        ->fields(
          [
            'table' => $item['table'],
            'foreign_key_id' => $item['foreign_key_id'],
            'created' => $item['created'],
          ]
        )
        ->execute();
      if (!$result) {
        $ifError = TRUE;
      }
    }

    if ($ifError) {
      return FALSE;
    }
    return TRUE;
  }

}
