<?php

namespace Drupal\list_packages;

use Composer\Semver\Constraint\MultiConstraint;
use Composer\Semver\Semver;
use Composer\Semver\VersionParser;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Asset\LibraryDiscovery;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DrupalKernel;
use Drupal\Core\Extension\ExtensionList;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ThemeHandler;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\testsuite\BaseFileService;

/**
 * Phpstan Tests Services.
 */
class ListPackagesResourceService extends BaseFileService {
  use StringTranslationTrait;

  /**
   * The '8.x-' prefix is used on contrib extension version numbers.
   *
   * @var string
   */
  const CORE_PREFIX = '8.x-';

  /**
   * The config interface.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The yaml service.
   *
   * @var Drupal\Core\Extension\ExtensionList
   */
  protected $extensionList;

  /**
   * The yaml service.
   *
   * @var Drupal\Component\Serialization\Yaml
   */
  protected $yaml;

  /**
   * The messenger service.
   *
   * @var Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The library array.
   *
   * @var array
   */
  protected $libraries;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The packagist client.
   *
   * @var \Drupal\list_packages\GetPackagistClient
   */
  protected $getPackagistClient;

  /**
   * The NPM Registery client.
   *
   * @var \Drupal\list_packages\GetNpmRegisteryClient
   */
  protected $getNpmRegisteryClient;

  /**
   * The Github client.
   *
   * @var \Drupal\list_packages\GetGithubClient
   */
  protected $getGithubClient;

  /**
   * The NPM Registery client.
   *
   * @var \Drupal\list_packages\GetDrupalClient
   */
  protected $getDrupalClient;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandler
   */
  protected $themeHandler;

  /**
   * The library discovery.
   *
   * @var \Drupal\Core\Asset\LibraryDiscovery
   */
  protected $libraryDiscovery;

  /**
   * ListPackagesResourceService constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Extension\ExtensionList $extensionList
   *   The yaml service.
   * @param \Drupal\Component\Serialization\Yaml $yaml
   *   The yaml service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The file system interface.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system interface.
   * @param \Drupal\list_packages\GetPackagistClient $getPackagistClient
   *   Get packagist info on a package.
   * @param \Drupal\list_packages\GetNpmRegisteryClient $getNpmRegisteryClient
   *   Get packagist info on a package.
   * @param \Drupal\list_packages\GetDrupalClient $getDrupalClient
   *   Get package.drupal info on a package.
   * @param \Drupal\list_packages\GetGithubClient $getGithubClient
   *   Get api.github.com info on a package.
   * @param \Drupal\Core\Extension\ModuleHandler $moduleHandler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ThemeHandler $themeHandler
   *   The theme handler service.
   * @param \Drupal\Core\Asset\LibraryDiscovery $libraryDiscovery
   *   The library discovery service.
   */
  public function __construct(
    ConfigFactoryInterface $configFactory,
    ExtensionList $extensionList,
    Yaml $yaml,
    MessengerInterface $messenger,
    FileSystemInterface $fileSystem,
    GetPackagistClient $getPackagistClient,
    GetNpmRegisteryClient $getNpmRegisteryClient,
    GetDrupalClient $getDrupalClient,
    GetGithubClient $getGithubClient,
    ModuleHandler $moduleHandler,
    ThemeHandler $themeHandler,
    LibraryDiscovery $libraryDiscovery,
  ) {
    $this->configFactory = $configFactory;
    $this->extensionList = $extensionList;
    $this->yaml = $yaml;
    $this->messenger = $messenger;
    $this->fileSystem = $fileSystem;
    $this->getPackagistClient = $getPackagistClient;
    $this->getNpmRegisteryClient = $getNpmRegisteryClient;
    $this->getDrupalClient = $getDrupalClient;
    $this->getGithubClient = $getGithubClient;
    $this->moduleHandler = $moduleHandler;
    $this->themeHandler = $themeHandler;
    $this->libraryDiscovery = $libraryDiscovery;
  }

  /**
   * Gets if the version is satisfied in the version consraint.
   *
   * @param string $version
   *   The version.
   * @param array $affectedVersions
   *   The version constraint.
   *
   * @return bool
   *   True or false.
   */
  private function getIfVersionInConstaint($version, $affectedVersions) {
    if ($affectedVersions == '*') {
      return TRUE;
    }

    if (str_starts_with($version, '8.x-')) {
      $version = preg_replace('/8\.x-/', '', $version, 1);
    }
    // Parsed constraints come back as one of Constraint for a single item
    // or MultiConstraint for multiple items.
    $versionparser = new VersionParser();
    $parsedConstraints = $versionparser->parseConstraints($version);
    if ($parsedConstraints instanceof MultiConstraint) {
      /** @var Composer\Semver\Constraint\MultiConstraint $parsedConstraints */
      foreach ($parsedConstraints->getConstraints() as $constraint) {
        if (Semver::satisfies($constraint->getVersion(), $affectedVersions)) {
          return TRUE;
        }
      }
    }
    else {
      /** @var Composer\Semver\Constraint\Constraint $parsedConstraints */
      if (Semver::satisfies($parsedConstraints->getVersion(), $affectedVersions)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Gets the vulnerability level.
   *
   * 0 = None
   * 1 = Has a vulnerability
   * 2 = Does not know because there is not enough information to get
   * a vulnerability report.
   * 3 = There is no version provided by the package information
   * but there was enough information to get a vulnerability report and
   * the package has known vulnerabilities.
   *
   * @param string $version
   *   The version.
   * @param array $vulnerabilitiesInfo
   *   The package vulnerabilities data.
   *
   * @return int
   *   The vulnerability level.
   */
  private function getIfVulnerabilities($version, $vulnerabilitiesInfo) {
    if (!empty($vulnerabilitiesInfo) && (!isset($version) || $version == NULL)) {
      return 1;
    }
    $vulnerabilities = 2;
    $ifVulnerabilities = FALSE;

    if (isset($vulnerabilitiesInfo)) {
      if (count($vulnerabilitiesInfo) > 0) {
        $ifVulnerabilities = TRUE;
        foreach ($vulnerabilitiesInfo as $item) {
          if ($this->getIfVersionInConstaint($version, $item['affectedVersions'])) {
            $vulnerabilities = 0;
          }
        }
      }
      else {
        $vulnerabilities = 3;
      }
    }

    if ($ifVulnerabilities && $vulnerabilities == 2) {
      $vulnerabilities = 3;
    }
    return $vulnerabilities;
  }

  /**
   * Get list of installed packages from the install.json file.
   *
   * @return array
   *   The array of package info.
   */
  public function getInstalledPackagesInfo() {

    $path = $this->getInstalledPath();
    if (!is_readable($path)) {
      return [];
    }

    $installedPackages = [];
    $installedData = json_decode(file_get_contents($path), TRUE);
    if (isset($installedData['packages'])) {
      foreach ($installedData['packages'] as $installedDataOfPackage) {
        $installedPackages["installed"][] = [
          "name" => $installedDataOfPackage["name"] ?? NULL,
          "direct-dependency" => FALSE,
          "homepage" => $installedDataOfPackage["homepage"] ?? NULL,
          "source" => $installedDataOfPackage['source']['url'] ?? NULL,
          "version" => $installedDataOfPackage["version"] ?? NULL,
          "latest" => $installedDataOfPackage["version"] ?? NULL,
          "latest-status" => "up-to-date",
          "description" => $installedDataOfPackage["description"] ?? NULL,
          "warning" => "",
          "abandoned" => FALSE,
        ];
      }
    }

    return $installedPackages;
  }

  /**
   * Gets composer show json.
   *
   * @return array
   *   The array of installed composer packages.
   */
  private function getComposerShowJson() {
    $config = $this->configFactory->get('testsuite.settings');
    if ($config->get('testsuite_if_composer') == 0) {
      $versions = $this->getInstalledPackagesInfo();
      // dd($versions);
      return $versions;
    }
    else {
      $statement = "composer show -l --format=json";
      $versions = $this->getData($statement);
      return Json::decode($versions);
    }
  }

  /**
   * Gets composer install path.
   *
   * @return string
   *   The path to installed.json.
   */
  private function getInstalledPath() {
    if (preg_match('/\bweb\b/', DRUPAL_ROOT)) {
      $docroot = dirname(DRUPAL_ROOT);
    }
    else {
      $docroot = DRUPAL_ROOT;
    }

    return $docroot . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'composer' . DIRECTORY_SEPARATOR . 'installed.json';
  }

  /**
   * Gets the array of composer packages less modules and installed libraries.
   *
   * @return array
   *   The array of installed composer packages.
   */
  public function getComposerPackages() {
    $packageArray = $this->getComposerShowJson();

    // Get root composer.json string.
    $composerJsonData = "";
    $composerJson = DRUPAL_ROOT . DIRECTORY_SEPARATOR . 'composer.json';
    if (file_exists($composerJson)) {
      $composerJsonData = file_get_contents($composerJson);
    }

    // Get all libraries that are direct dependents.
    $libraryCount = 0;
    $drupalModuleCount = 0;
    $libraryDependencies = [];
    if (isset($packageArray['installed'])) {
      foreach ($packageArray['installed'] as $info) {
        $vendorName = explode("/", $info['name'])[0];
        $packageName = explode("/", $info['name'])[1];
        if ($info['direct-dependency'] == TRUE && !preg_match('/\bdrupal\b/', $info['name']) && ($packageName != "" && $composerJsonData != "" && strpos($composerJsonData, $packageName) == FALSE)) {
          $libraryDependencies[] = $info;
        }
      }

      foreach ($packageArray['installed'] as $info) {
        $vendorName = explode("/", $info['name'])[0];
        $packageName = explode("/", $info['name'])[1];

        // Remove add drupal modules and libraries.
        $isLibrary = FALSE;
        $isDrupalModule = FALSE;
        if (strtolower($vendorName) == 'drupal') {
          $isDrupalModule = TRUE;
          $drupalModuleCount++;
        }
        foreach ($libraryDependencies as $libraryInfo) {
          if ($libraryInfo['name'] == $info['name']) {
            $isLibrary = TRUE;
            $libraryCount++;
          }
        }

        if (!$isLibrary && !$isDrupalModule) {
          $packageData[$info['name']] = $this->getPackagistInformation($info, $vendorName, $packageName);
        }
      }
    }
    else {
      $drupalLink = Link::fromTextAndUrl(
        $this->t('drupal.org/download'),
        Url::fromUri('https://www.drupal.org/download')
      )->toString();
      $this->messenger->addWarning(Markup::create('You have returned an empty list of packages. You must use a Drupal version either installed with composer or downloaded as a zip from ' . $drupalLink));
    }

    if ($libraryCount > 0) {
      $librariesLink = Link::createFromRoute('Library Report', 'list_packages.installed_libraries')
        ->toString()
        ->getGeneratedLink();
      $this->messenger->addStatus(Markup::create('There are ' . $libraryCount . ' libraries installed that are direct dependencies of one or more modules. There are a total of ' . count($this->getLibrariesDirectories()) . ' libraries installed. Please refer to the ' . $librariesLink . ' page.'));
    }

    return $packageData;
  }

  /**
   * Gets the array of libraries directories.
   *
   * @return array
   *   The array of installed libraries by directory.
   */
  private function getLibrariesDirectories() {
    $searchdir = [];
    $config = DrupalKernel::findSitePath(\Drupal::request());
    if ($profile = \Drupal::installProfile()) {
      $profile_path = $this->extensionList->getPath($profile);
      $searchdir[] = "$profile_path/libraries";
    };
    $searchdir[] = 'sites/all/libraries';
    $searchdir[] = 'libraries';
    $searchdir[] = "$config/libraries";
    $directories = [];
    $nomask = ['CVS'];
    foreach ($searchdir as $dir) {
      if (is_dir($dir) && $handle = opendir($dir)) {
        while (FALSE !== ($file = readdir($handle))) {
          if (!in_array($file, $nomask) && $file[0] != '.') {
            if (is_dir("$dir/$file")) {
              $directories[$file] = "$dir/$file";
            }
          }
        }
        closedir($handle);
      }
    }

    return $directories;
  }

  /**
   * Gets the latest version.
   *
   * @param array $packageVersions
   *   The versions returned from packagist.org.
   *
   * @return string
   *   The latest version.
   */
  private function getLatestVersion($packageVersions) {
    $versions = array_keys($packageVersions);
    $vers = [];
    foreach ($versions as $version) {
      if (preg_match('/^v?([0-9]+\.?)+$/', $version)) {
        $vers[] = $version;
      }
    }
    $newVersions = [];
    foreach ($vers as $ver) {
      $newVersions[] = str_replace("v", "", $ver);
    }
    usort($newVersions, 'version_compare');

    $versionString = end($newVersions);
    $endVersion = "";
    foreach ($versions as $version) {
      if (str_contains($version, $versionString)) {
        $endVersion = $version;
      }
    }

    return $endVersion;
  }

  /**
   * If github url.
   *
   * @param string $url
   *   An url that can be pulled apart for vendor and product names.
   *
   * @return bool
   *   true or false.
   */
  private function isGithubUrl($url) {
    if (preg_match('/\bgithub\.com\b/', $url)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Gets the vendor and product from github url.
   *
   * @param string $url
   *   An url that can be pulled apart for vendor and product names.
   *
   * @return array
   *   The [vendor, product].
   */
  private function getGithubVendorAndProduct($url) {
    $parts = preg_split('/\bgithub\.com\b/', $url);
    if (isset($parts[1])) {
      $trimmed = trim($parts[1], "/");
      $pathParts = explode("/", $trimmed);
      if (isset($pathParts[1])) {
        if (isset($pathParts[1]) && preg_match('/[#]{1}/', $pathParts[1])) {
          $pathParts[1] = explode("#", $pathParts[1])[0];
        }
      }
      if (isset($pathParts[1])) {
        if (isset($pathParts[1]) && preg_match('/[\.]{1}/', $pathParts[1])) {
          $pathParts[1] = explode(".", $pathParts[1])[0];
        }
      }
    }

    return [
      'vendor' => $pathParts[0] ?? "",
      'product' => $pathParts[1] ?? "",
    ];
  }

  /**
   * Get core libraries.
   *
   * @return array
   *   Returns an array of libraries.
   */
  public function getCoreLibraries(): array {
    $libraries = $this->libraryDiscovery->getLibrariesByExtension('core');
    foreach ($libraries as $extension_name => $extension) {
      $newLibraries[$extension_name]['extension_name'] = $extension_name;
      if (isset($extension['version'])) {
        $newLibraries[$extension_name]['version'] = $extension['version'];
      }
      if (isset($extension['remote']) && $this->isGithubUrl($extension['remote'])) {
        $values = $this->getGithubVendorAndProduct($extension['remote']);
        $newLibraries[$extension_name]['vendor'] = $values['vendor'];
        $newLibraries[$extension_name]['product'] = $values['product'];
        $newLibraries[$extension_name]['remote'] = $extension['remote'];
      }

      $newLibraries[$extension_name]['css'] = $extension['css'];
      if (isset($extension['css']) && count($extension['css']) > 0) {
        foreach ($extension['css'] as $index => $cssLib) {
          if (preg_match('/^https?/', $cssLib['data'])) {
            $newLibraries[$extension_name]['css'][$index]['internal'] = FALSE;
            if (isset($extension['remote']) && $this->isGithubUrl($extension['remote'])) {
              $newLibraries[$extension_name]['packagist'] = $this->getPackagistInformation(
                $newLibraries[$extension_name],
                $newLibraries[$extension_name]['vendor'],
                $newLibraries[$extension_name]['product']
              );
            }
          }
          else {
            $newLibraries[$extension_name]['css'][$index]['internal'] = TRUE;
          }
        }
      }

      $newLibraries[$extension_name]['js'] = $extension['js'];
      if (isset($extension['js']) && count($extension['js']) > 0) {
        foreach ($extension['js'] as $index => $jsLib) {
          if (preg_match('/^https?/', $jsLib['data'])) {
            $newLibraries[$extension_name]['js'][$index]['internal'] = FALSE;
            if (isset($extension['remote']) && $this->isGithubUrl($extension['remote'])) {
              $newLibraries[$extension_name]['packagist'] = $this->getPackagistInformation(
                $newLibraries[$extension_name],
                $newLibraries[$extension_name]['vendor'],
                $newLibraries[$extension_name]['product']
              );
            }
          }
          else {
            $newLibraries[$extension_name]['js'][$index]['internal'] = TRUE;
          }
        }
      }
    }

    // dd($newLibraries);
    return $newLibraries;
  }

  /**
   * Get a cleaned list of vulnerabilities from drupal API.
   *
   * @param array $vulnerabilities
   *   Array of vulnerabilities.
   *
   * @return array
   *   Returns an array of vulnerabilities.
   */
  public function createVulnerabilityArrayDrupal($vulnerabilities) {
    $vulnerabilityArray = [];
    foreach ($vulnerabilities as $vulnerability) {
      $vulnerabilityArray[] = [
        'packageName' => $vulnerability["packageName"],
        'affectedVersions' => $vulnerability["affectedVersions"],
        'first_patched_version' => NULL,
      ];
    }
    return $vulnerabilityArray;
  }

  /**
   * Get a cleaned list of vulnerabilities from github API.
   *
   * @param array $vulnerabilities
   *   Array of vulnerabilities.
   *
   * @return array
   *   Returns an array of vulnerabilities.
   */
  public function createVulnerabilityArrayGithub($vulnerabilities) {
    $vulnerabilityArray = [];
    if (is_array($vulnerabilities)) {
      foreach ($vulnerabilities as $vulnerability) {
        foreach ($vulnerability["vulnerabilities"] as $affectedVersions) {
          $vulnerabilityArray[] = [
            'packageName' => $affectedVersions["package"]["name"],
            'affectedVersions' => $affectedVersions["vulnerable_version_range"],
            'first_patched_version' => $affectedVersions["first_patched_version"],
          ];
        }
      }
    }

    return $vulnerabilityArray;
  }

  /**
   * Get all libraries.
   *
   * @return array
   *   Returns an array of libraries.
   */
  public function getModulesThemesAndTheirLibraries(): array {
    // phpcs:ignore
    $root = \Drupal::root();
    $drupalShowModules = [];

    // Runs composer show -l --format=json
    // so the information from it can be used.
    $packageArray = $this->getComposerShowJson();
    if (isset($packageArray['installed'])) {
      foreach ($packageArray['installed'] as $info) {
        $drupalShowModules[explode("/", $info['name'])[1]] = $info;
      }
    }
    else {
      $drupalLink = Link::fromTextAndUrl(
        $this->t('drupal.org/download'),
        Url::fromUri('https://www.drupal.org/download')
          )->toString();
      $this->messenger->addWarning(Markup::create('The installation of Drupal you are using is unsupported. There is not enough information to get all available results and the results might be misleading. You must use a Drupal version either installed with composer or downloaded as a zip from ' . $drupalLink));
    }

    // Get modules and theme. Then merges content.
    $modules = $this->moduleHandler->getModuleList();
    $themes = $this->themeHandler->rebuildThemeData();
    $mods = array_merge($modules, $themes);

    // Iterate over modules and themes.
    $githubApiCount = 0;
    $githubApiFailed = FALSE;
    foreach ($mods as $extension_name => $extension) {
      if ($extension->getType() != "profile") {

        // Load the composer show -l into show-info and sets the version.
        $libraries[$extension->getType()][$extension_name]['show-info'] = $drupalShowModules[$extension_name] ?? "No Info";
        $version = $libraries[$extension->getType()][$extension_name]['show-info']['version'] ?? NULL;

        // Gets the module or theme information from module.info.yml file.
        $info_file = $extension->getPath() . '/' . $extension_name . '.info.yml';
        if (is_file($root . '/' . $info_file)) {
          $infoDecoded = $this->yaml->decode(file_get_contents($root . '/' . $info_file));
          $libraries[$extension->getType()][$extension_name]['info'] = $infoDecoded;
        }

        if ($version == NULL) {
          $version = $infoDecoded['version'] ?? NULL;
        }

        // Sets the path to the module or theme.
        $libraries[$extension->getType()][$extension_name]['info']['path'] = $this->fileSystem->realPath($root . DIRECTORY_SEPARATOR . $extension->getPath());

        // Gets the library information from module.libraries.yml file.
        $library_file = $extension->getPath() . '/' . $extension_name . '.libraries.yml';
        if (is_file($root . '/' . $library_file)) {
          $libraries[$extension->getType()][$extension_name]['libraries'] = $this->libraryDiscovery->getLibrariesByExtension($extension_name);
          foreach ($libraries[$extension->getType()][$extension_name]['libraries'] as $name => $items) {
            foreach ($items as $type => $item) {
              if ($type == "css" || $type == "js") {
                foreach ($item as $index => $lib) {

                  // Sets the module or theme library path.
                  $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['path'] = $lib['data'];

                  // Checks whether the library is external.
                  if (preg_match('/^https?/', $lib['data'])) {
                    // dd($name, $items);.
                    $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['internal'] = FALSE;
                    if ($libraries[$extension->getType()][$extension_name]['show-info'] == "No Info") {
                      $scriptName = $extension_name;
                    }
                    else {
                      $scriptName = $libraries[$extension->getType()][$extension_name]['show-info']['name'];
                    }

                    $vendor = "";
                    $product = "";
                    if (isset($items['remote']) && $this->isGithubUrl($items['remote'])) {
                      $values = $this->getGithubVendorAndProduct($items['remote']);
                      $vendor = $values['vendor'];
                      $product = $values['product'];
                    }
                    elseif (isset($items["license"]["url"]) && $this->isGithubUrl($items["license"]["url"])) {
                      $values = $this->getGithubVendorAndProduct($items["license"]["url"]);
                      $vendor = $values['vendor'];
                      $product = $values['product'];
                    }
                    $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vendor'] = $vendor;
                    $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['product'] = $product;
                    $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vulnerabilities'] = 2;

                    // If a drupal package check drupal.org.
                    if (preg_match('/\bdrupal\b/', $scriptName)) {
                      $drupalInformation = $this->checkDrupalVulnerability($extension_name, $version);
                      $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['warning'] = $drupalInformation[0];
                      $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vulnerabilities'] = $drupalInformation[1];
                    }
                    else {
                      if (isset($items['remote']) && $this->isGithubUrl($items['remote'])) {
                        $values = $this->getGithubVendorAndProduct($items['remote']);
                        $product = $values['product'];
                        $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['packagist'] = $this->getPackagistInformation(
                          $libraries[$extension->getType()][$extension_name]['show-info'],
                          $values['vendor'],
                          $values['product']
                        );
                      }
                    }

                    // If returns no vulnerabilities run it through github.
                    if ($libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vulnerabilities'] >= 2 && $product) {
                      $results = $this->getGithubClient->getAdvisoryInfo($product);
                      $githubApiCount++;

                      if ($results['status'] == TRUE) {
                        $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vulnerabilities'] = 3;
                        if (is_array($results['message']) && count($results['message']) > 0) {
                          $vulnerabilityArray = $this->createVulnerabilityArrayGithub($product, $results['message']);
                          $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['vulnerabilities'] = $this->getIfVulnerabilities($version, $vulnerabilityArray);
                        }
                      }
                      else {
                        if (preg_match('/\brate\slimit\sexceeded\b/', $results['message'])) {
                          $githubApiFailed = TRUE;
                        }
                      }
                    }
                  }
                  else {
                    $libraries[$extension->getType()][$extension_name]['libraries'][$name][$type][$index]['internal'] = TRUE;
                  }
                }
              }
            }
          }
        }

        // Check drupal for vulnerabilities.
        $drupalInformation = $this->checkDrupalVulnerability($extension_name, $version);
        $libraries[$extension->getType()][$extension_name]['info']['warning'] = $drupalInformation[0];
        $libraries[$extension->getType()][$extension_name]['info']['vulnerabilities'] = $drupalInformation[1];
      }
    }

    $libraries['github_api_failed'] = $githubApiFailed;
    $libraries['github_api_calls'] = $githubApiCount;
    return $libraries;
  }

  /**
   * Gets module otr them content.
   *
   * @param string $extension_name
   *   The extension name.
   * @param object $version
   *   The version.
   *
   * @return array
   *   The module or theme vulnerability data.
   */
  private function checkDrupalVulnerability($extension_name, $version) {
    $libraries['vulnerabilities'] = $this->getDrupalClient->getVulnerabilities('drupal', $extension_name);
    $vulnerabilitiesApiFailed = "";
    $vulnerabilities = 2;
    if ($libraries['vulnerabilities']['status'] && empty($libraries['vulnerabilities']['advisories'])) {
      $vulnerabilities = 3;
    }
    elseif ($libraries['vulnerabilities']['status'] && count($libraries['vulnerabilities']['advisories']) > 0) {
      if (isset($libraries['vulnerabilities']['advisories']["drupal/" . $extension_name])) {
        $vulnerabilities = $this->getIfVulnerabilities($version, $libraries['vulnerabilities']['advisories']["drupal/" . $extension_name]);
      }
    }
    else {
      $vulnerabilitiesApiFailed = $libraries['vulnerabilities']['message'];
    }
    // $libraries['info']['vulnerabilities'] = $vulnerabilities;
    // $libraries['info']['warning'] = $vulnerabilitiesApiFailed;
    return [$vulnerabilitiesApiFailed, $vulnerabilities];
  }

  /**
   * Gets packagist data and vulnerabilities.
   *
   * @param array $info
   *   The package data.
   * @param string $vendorName
   *   The vendor name.
   * @param string $packageName
   *   The package name.
   *
   * @return array
   *   The package data to be stored in the database.
   */
  private function getPackagistInformation($info, $vendorName, $packageName) {

    // Get packagist information & vulnerabilities.
    $packageInfo = $this->getPackagistClient->getPackagistInfo($vendorName, strtolower($packageName));
    $vulnerabilitiesApiFailed = "";
    $vulnerabilities = 2;
    $vulnerabilityInfo = $this->getPackagistClient->getVulnerabilities($vendorName, strtolower($packageName));

    if ($vulnerabilityInfo['status']) {
      if (isset($vulnerabilityInfo['advisories'][$vendorName . "/" . strtolower($packageName)])) {
        $version = $info['version'] ?? NULL;
        $vulnerabilities = $this->getIfVulnerabilities($version, $vulnerabilityInfo['advisories'][$vendorName . "/" . strtolower($packageName)]);
      }
    }
    else {
      $vulnerabilitiesApiFailed = $vulnerabilityInfo['message'];
    }

    // Build return data array.
    $packageData['status'] = $packageInfo['status'];
    $packageData['type'] = 'composer';
    $packageData['name'] = $info['name'] ?? "No Name";
    $packageData['vendor'] = $vendorName;
    $packageData['product'] = $packageName;
    $packageData['version'] = $info['version'] ?? "ANY";
    $packageData['if_vulnerability'] = $vulnerabilities;
    if ($packageInfo['status']) {
      $latest = $this->getLatestVersion($packageInfo['package']['versions']);
      $latestVersion = $packageInfo['package']['versions'][$latest];
      $packageData['latest'] = $latest;
      $packageData['homepage'] = $latestVersion['homepage'];
      $packageData['source'] = $latestVersion['source']['url'];
      $packageData['description'] = $latestVersion['description'] ?? "";
      $packageData['warning'] = $vulnerabilitiesApiFailed;
    }
    else {
      $packageData['latest'] = "ANY";
      $packageData['homepage'] = $info['homepage'] ?? "";
      $packageData['source'] = "";
      $packageData['description'] = $info['description'] ?? "";
      $packageData['warning'] = $packageInfo['message'] . " " . $vulnerabilitiesApiFailed;
    }
    $packageData['abandoned'] = $info['abandoned'] ?? FALSE;
    $packageData['latest-status'] = $info['latest-status'] ?? "";

    return $packageData;
  }

  /**
   * Gets Npm Registory data and vulnerabilities.
   *
   * @param string $name
   *   The package name.
   * @param array $installedPackageInfo
   *   The package info.
   *
   * @return array
   *   The package data to be stored in the database.
   */
  private function getNpmRegistoryInfo($name, $installedPackageInfo) {
    $vulnerabilitiesApiFailed = "";
    $vulnerabilities = 2;

    // Check package for vulnerabilities.
    $vulnerabilityInfo = $this->getNpmRegisteryClient->getVulnerabilities($installedPackageInfo);
    if ($vulnerabilityInfo['status']) {
      if (isset($vulnerabilityInfo['metadata']['vulnerabilities'])) {
        $vulnerabilities = 3;
        foreach ($vulnerabilityInfo['metadata']['vulnerabilities'] as $item) {
          if ($item > 0) {
            $vulnerabilities = 0;
          }
        }
      }
    }
    else {
      $vulnerabilitiesApiFailed = $vulnerabilityInfo['message'];
    }

    // Get vendor and package name.
    $vendor = "";
    $product = "";
    $homepage = $installedPackageInfo['homepage'] ?? "";
    if (isset($homepage) && $this->isGithubUrl($homepage)) {
      $values = $this->getGithubVendorAndProduct($homepage);
      $vendor = $values['vendor'];
      $product = $values['product'];
    }

    $source = "";
    if (isset($installedPackageInfo['repository']) && is_array($installedPackageInfo['repository'])) {
      $source = $installedPackageInfo['repository']['url'];
    }
    elseif (isset($installedPackageInfo['repository'])) {
      $source = $installedPackageInfo['repository'];
    }

    if ($vendor == "" && $product == "" && $source != "") {
      if (isset($source) && $this->isGithubUrl($source)) {
        $values = $this->getGithubVendorAndProduct($source);
        $vendor = $values['vendor'];
        $product = $values['product'];
      }
    }

    // Get latest version information.
    $packageInfo = $this->getNpmRegisteryClient->getNpmRegisteryInfo($name, 'latest');

    // Build return data array.
    $packageData['status'] = $packageInfo['status'];
    $packageData['type'] = 'package';
    $packageData['name'] = $installedPackageInfo['name'];
    $packageData['vendor'] = $vendor ? $vendor : $installedPackageInfo['name'];
    $packageData['product'] = $product ? $product : $installedPackageInfo['name'];
    $packageData['homepage'] = $homepage;
    $packageData['source'] = $source;
    $packageData['description'] = $installedPackageInfo['description'] ?? "";
    $packageData['if_vulnerability'] = $vulnerabilities;
    $packageData['version'] = $installedPackageInfo['version'] ?? "ANY";
    $packageData['latest-status'] = "";
    $packageData['abandoned'] = FALSE;
    if ($packageInfo['status']) {
      $packageData['latest'] = $packageInfo['version'];
      $packageData['warning'] = $vulnerabilitiesApiFailed;
    }
    else {
      $packageData['latest'] = "ANY";
      $packageData['warning'] = $packageInfo['message'] . " " . $vulnerabilitiesApiFailed;
    }

    return $packageData;
  }

  /**
   * Gets the array of libraries packages.
   *
   * @return array
   *   The array of installed libraries.
   */
  public function getLibrariesPackages() {
    $libraries = [];

    $githubApiFailed = FALSE;
    $githubApiCount = 0;
    $vulnerability_count = 0;
    foreach ($this->getLibrariesDirectories() as $name => $info) {
      $path = $this->fileSystem->realPath(DRUPAL_ROOT . DIRECTORY_SEPARATOR . $info);
      if (file_exists($path)) {
        $composer = $this->fileSystem->realPath($path . DIRECTORY_SEPARATOR . 'composer.json');
        $package = $this->fileSystem->realPath($path . DIRECTORY_SEPARATOR . 'package.json');
        if (file_exists($composer)) {
          $read_file = file_get_contents($composer);
          $info = json_decode($read_file, TRUE);
          $vendorName = explode("/", $info['name'])[0];
          $packageName = explode("/", $info['name'])[1];
          $libraries[$name] = $this->getPackagistInformation($info, $vendorName, $packageName);
          if ($libraries[$name]['status'] == FALSE) {
            $libraries[$name] = $this->getPackagistInformation($info, $vendorName, $name);
          }
        }
        elseif (file_exists($package)) {
          $read_file = file_get_contents($package);
          $packageArray = json_decode($read_file, TRUE);
          $libraries[$name] = $this->getNpmRegistoryInfo($packageArray['name'], $packageArray);

          if ($libraries[$name]['if_vulnerability'] > 1) {
            $vulnerabilities = 2;

            // Get vendor and package name.
            $vendor = "";
            $product = "";
            $homepage = $packageArray['homepage'] ?? "";
            if (isset($homepage) && $this->isGithubUrl($homepage)) {
              $values = $this->getGithubVendorAndProduct($homepage);
              $vendor = $values['vendor'];
              $product = $values['product'];
            }

            $source = "";
            if (isset($packageArray['repository']) && is_array($packageArray['repository'])) {
              $source = $packageArray['repository']['url'];
            }
            elseif (isset($packageArray['repository'])) {
              $source = $packageArray['repository'];
            }

            if ($vendor == "" && $product == "" && $source != "") {
              if (isset($source) && $this->isGithubUrl($source)) {
                $values = $this->getGithubVendorAndProduct($source);
                $vendor = $values['vendor'];
                $product = $values['product'];
              }
            }

            // Get github information.
            $githubAdvisories = $this->getGithubClient->getAdvisoryInfo($packageArray['name']);

            $githubApiCount++;
            if ($githubAdvisories['status']) {
              if (isset($githubAdvisories['message'])) {
                $vulnerabilities = 3;
                foreach ($githubAdvisories['message'] as $vulnerability) {
                  foreach ($vulnerability['vulnerabilities'] as $item) {
                    if ($item["package"]["name"] == $product) {
                      if ($this->getIfVersionInConstaint($packageArray['version'], $item["vulnerable_version_range"])) {
                        $vulnerabilities = 0;
                        $vulnerability_count = count($githubAdvisories['message']);
                      }
                    }
                  }
                }
              }
            }
            else {
              if (preg_match('/\brate\slimit\sexceeded\b/', $githubAdvisories['message'])) {
                $githubApiFailed = TRUE;
              }
            }
            $libraries[$name]['if_vulnerability'] = $vulnerabilities;
          }
        }
        else {
          $libraries[$name]['type'] = 'other';
          $libraries[$name]['name'] = $name;
          $libraries[$name]['vendor'] = $name;
          $libraries[$name]['product'] = $name;
          $libraries[$name]['version'] = "ANY";
          $libraries[$name]['latest'] = "ANY";
          $libraries[$name]['homepage'] = "";
          $libraries[$name]['source'] = "";
          $libraries[$name]['description'] = "";
          $libraries[$name]['if_vulnerability'] = 2;
          $libraries[$name]['warning'] = "";
          $libraries[$name]['latest-status'] = "";
          $libraries[$name]['abandoned'] = FALSE;
        }
        $libraries[$name]['path'] = $path;
        $libraries[$name]['vulnerability_count'] = $vulnerability_count;
      }
    }

    ksort($libraries);

    $libraries['github_api_failed'] = $githubApiFailed;
    $libraries['github_api_calls'] = $githubApiCount;
    // dd($libraries);
    return $libraries;
  }

}
